Package: goseq
Title: Gene Ontology analyser for RNA-seq and other length biased data
Version: 1.65.0
Date: 2025-07-22
Authors@R: 
    c(
        person(
            given = "Matthew", family = "Young", role = c("aut"), 
            email = "my4@sanger.ac.uk"
        ),
        person(
            given = "Nadia", family = "Davidson", role = c("aut"), 
            email = "nadia.davidson@mcri.edu.au"
        ),
        person(
            given = "Federico", family = "Marini", role = c("ctb", "cre"), 
            email = "marinif@uni-mainz.de", comment = c(ORCID = "0000-0003-3252-7758")
        )
    )
Description: Detects Gene Ontology and/or other user defined categories
    which are over/under represented in RNA-seq data.
Depends: R (>= 2.11.0), BiasedUrn, geneLenDataBase (>= 1.9.2)
Imports: mgcv, graphics, stats, utils, AnnotationDbi, GO.db,
        BiocGenerics, methods, rtracklayer, GenomicFeatures, Seqinfo
Suggests: edgeR, org.Hs.eg.db
License: LGPL (>= 2)
URL: https://github.com/federicomarini/goseq
BugReports: https://github.com/federicomarini/goseq/issues
biocViews: ImmunoOncology, Sequencing, GO, GeneExpression,
        Transcription, RNASeq, DifferentialExpression, Annotation,
        GeneSetEnrichment, KEGG, Pathways, Software
RoxygenNote: 7.3.1
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/goseq
git_branch: devel
git_last_commit: 8a3353b
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:00:25 UTC; biocbuild
Author: Matthew Young [aut],
  Nadia Davidson [aut],
  Federico Marini [ctb, cre] (ORCID:
    <https://orcid.org/0000-0003-3252-7758>)
Maintainer: Federico Marini <marinif@uni-mainz.de>

# Generated by roxygen2: do not edit by hand

export(getgo)
export(getlength)
export(goseq)
export(makespline)
export(nullp)
export(plotPWF)
export(supportedOrganisms)
import(geneLenDataBase)
importFrom(AnnotationDbi,as.list)
importFrom(AnnotationDbi,select)
importFrom(BiasedUrn,dWNCHypergeo)
importFrom(BiasedUrn,pWNCHypergeo)
importFrom(BiocGenerics,relist)
importFrom(BiocGenerics,toTable)
importFrom(GO.db,GO.db)
importFrom(GenomicFeatures,transcriptLengths)
importFrom(Seqinfo,`genome<-`)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(methods,show)
importFrom(mgcv,Predict.matrix)
importFrom(mgcv,gam)
importFrom(mgcv,mono.con)
importFrom(mgcv,pcls)
importFrom(mgcv,s)
importFrom(mgcv,smoothCon)
importFrom(rtracklayer,browserSession)
importFrom(rtracklayer,getTable)
importFrom(rtracklayer,ucscGenomes)
importFrom(rtracklayer,ucscTableQuery)
importFrom(stats,approx)
importFrom(stats,binomial)
importFrom(stats,dhyper)
importFrom(stats,median)
importFrom(stats,phyper)
importFrom(stats,runif)
importFrom(utils,data)
importFrom(utils,installed.packages)
importFrom(utils,tail)

# goseq

<!-- badges: start -->
[![R-CMD-check](https://github.com/federicomarini/goseq/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/federicomarini/goseq/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

The `goseq` package provides a method to detect Gene Ontology and/or other user 
defined categories which are over/under represented in RNA-seq data.

`goseq` can be found on Bioconductor
(<https://www.bioconductor.org/packages/goseq>).

## Installation

You can install the version of `goseq` which is on Bioconductor with these commands:

``` r
if (!require("BiocManager")) {
  install.packages("BiocManager")
}
BiocManager::install("goseq")
```

Alternatively, you can install the development version of `goseq` from GitHub with:

``` r
library("remotes")
remotes::install_github("federicomarini/goseq")
```

#############################################################################
# Description: Reverses the direction of a bidirectional mapping, removes an entries pointing to NULL
# Notes:
# Author: Matthew Young
# Date Modified: 13/12/2010

reversemapping <- function(map) {
  tmp <- unlist(map, use.names = FALSE)
  names(tmp) <- rep(names(map), times = as.numeric(summary(map)[, 1]))
  return(split(names(tmp), as.vector(tmp)))
}

#' Prints progress through a loop
#'
#' @param total total number of iterations
#' @param count current iteration
#' @param i index of the loop
#'
#' @return message indicating the progress
pp <- function(total, count, i = i) {
  if (missing(count)) {
    count <- evalq(i, envir = parent.frame())
  }
  if (missing(total)) {
    total <- evalq(stop, envir = parent.frame())
  }
  message(round(100 * (count / total)), "%   \r")
}

#' Androgen stimulation of prostate cancer Cell lines.
#' 
#' This data set gives the RNA-seq data from an experiment measuring the
#' effects of androgen stimulation on prostate cancer.  Information is given
#' about all (ENSEMBL) genes for which there was at least one mapping read in
#' either the treated or untreated RNA-seq experiment.  The \code{edgeR}
#' package was used to determine which genes were differentially expressed.
#' The details of the analysis can be found in the \code{goseq} vignette.
#' 
#' 
#' @name genes
#' @docType data
#' @format A named vector of ENSEMBL genes, with 1 representing differential
#' expression.
#' @references Li, H., Lovci, M. T., Kwon, Y. S., Rosenfeld, M. G., Fu, X. D.,
#' Yeo, G. W. (2008) \emph{Determination of tag density required for digital
#' transcriptome analysis: application to an androgen-sensitive prostate cancer
#' model} Proceedings of the National Academy of Sciences of the United States
#' of America Date: Dec 23 Vol: 105 Issue: 51 Pages: 20179-84
#' @source Determination of tag density required for digital transcriptome
#' analysis: application to an androgen-sensitive prostate cancer model, 2008,
#' Li et. al.
#' @keywords datasets
#' @examples
#' 
#' data(genes)
#' head(genes)
#' 
NULL

#  ZZZ.R

.onAttach <- function(libname, pkgname) {
  if (.Platform$OS.type == "windows" && .Platform$GUI == "Rgui") {
    utils::winMenuAddItem("Vignettes", "goseq", "shell.exec(system.file(\"doc\",\"goseq.pdf\",package=\"goseq\"))")
  }
}

# These two variables are required for automatic fetching of categories to function.  Their purpose is to take the UCSC genome and gene ID values given when looking up length data and convert them to the names used for the same organism and gene identifier in the organism packages.

# Mappings that are primarily required by getgo, the purpose of this is to convert the UCSC genome IDs, to the bioconductor organism names, e.g. "mm"->"org.Mm."
.ORG_PACKAGES <- paste("org.", c("Ag.eg", "At.tair", "Bt.eg", "Ce.eg", "Cf.eg", "Dm.eg", "Dr.eg", "EcK12.eg", "EcSakai.eg", "Gg.eg", "Hs.eg", "Mm.eg", "Mmu.eg", "Pf.plasmo", "Pt.eg", "Rn.eg", "Sc.sgd", "Ss.eg", "Xl.eg"), sep = "")
names(.ORG_PACKAGES) <- c("anoGam", "Arabidopsis", "bosTau", "ce", "canFam", "dm", "danRer", "E. coli K12", "E. coli Sakai", "galGal", "hg", "mm", "rheMac", "Malaria", "panTro", "rn", "sacCer", "susScr", "xenTro")

# These are the only formats supported by getgo at the moment, the purpose is to convert the USCC gene
# ID formats, to the shorthand used by the bioconductor organism packages, .e.g. "refGene"->"ENSEMBL"
.ID_MAP <- c("eg", "eg", "ENSEMBL", "SYMBOL", "sgd", "plasmo", "tair")
names(.ID_MAP) <- c("knownGene", "refGene", "ensGene", "geneSymbol", "sgd", "plasmo", "tair")

# Below are the exceptions to the function name for gene to go term mappings
.ORG_GOMAP_FUNCTION <- c("GO2ALLEGS", "GO2ALLTAIRS", "GO2ALLORFS", "GO2ALLORFS")
names(.ORG_GOMAP_FUNCTION) <- c("default", "org.At.tair", "org.Pf.plasmo", "org.Sc.sgd")

# TxDb Length databases
.TXDB_ORGS <- c("ce6", "dm3", "hg18", "hg19", "hg38", "mm10", "mm9", "rn4", "rn5", "sacCer2", "sacCer3")

#' Supported Organisms
#' 
#' Lists which genomes and gene ids are supported for gene lengths and for gene
#' ontology
#' 
#' Goseq allows a user to provide their own bias data (usually gene lengths)
#' and/or gene categories (usually gene ontologies), but goseq also provides
#' this data automatically for many commonly used species. This function lists
#' which genome and gene ids are automatically supported by goseq. The first to
#' third columns list the genomes, gene ids, and gene id descriptions
#' respectively. The fourth column indicates whether this combination of genome
#' and id are available in the geneLengthDataBase. If a particular combination
#' is absent, goseq may still automatically fetch the gene lengths from either
#' a TxDB annotation package (must be installed) or download the data from
#' UCSC. For example gene lengths for \code{hg38} are not supported in
#' \code{geneLengthDataBase} but may be fetched by these other means. However,
#' this is not always the case. The final column indicates if GO terms will be
#' automatically fetched for the genome and id combination.  Goseq relies on an
#' \code{org} annotation package (e.g. \code{org.Hs.eg.db}) existing for the
#' organism. In general, if either the lengths or GO terms are not supported,
#' the user must enter this information manually.
#' 
#' @return A data.frame containing supported genomes and gene ids
#' 
#' @author Nadia Davidson \email{nadia.davidson@@mcri.edu.au}
#' 
#' @export
#' 
#' @examples
#' supportedOrganisms()
#' 
supportedOrganisms <- function() {
  requireNamespace("rtracklayer")

  # Start by getting the geneLenDataBase supported genomes and geneIDs
  geneIDs <- supportedGeneIDs()

  # The names of the supported geneIDs is hard-coded so we can remove things like
  # "refGene" which actually correspond to an Entrez gene ID in geneLenDataBase
  uniqueIDs <- c("knownGene", "vegaGene", "ensGene", "geneSymbol")

  geneIDs <- geneIDs[geneIDs$tablename %in% uniqueIDs, ]

  # extract the corresponding genome IDs and rearrange
  genomes <- strsplit(geneIDs$AvailableGenomes, ",")
  id <- rep(geneIDs$tablename, sapply(genomes, length))
  description <- (rep(geneIDs$GeneID, sapply(genomes, length)))
  length_supported <- rep(TRUE, length(id))
  genomes <- unlist(genomes)
  tab_temp <- data.frame(genomes, id, description, length_supported)

  ## Is there GO annotation package for that genome and gene ID?
  genome_supported_go <- gsub("[0-9]+", "", genomes) %in% names(.ORG_PACKAGES)
  id_supported_go <- id %in% names(.ID_MAP)

  go_supported <- genome_supported_go & id_supported_go
  tab_temp <- data.frame(tab_temp, go_supported)
  colnames(tab_temp) <- c(
    "Genome", "Id", "Id Description", "Lengths in geneLeneDataBase",
    "GO Annotation Available"
  )

  ### Add in the newer genomes which aren't supported for length, but are for GO terms
  base <- unfactor(ucscGenomes())$db
  other_genomes <- base[!base %in% genomes]
  new_genomes <- other_genomes[gsub("[0-9]+", "", other_genomes) %in% names(.ORG_PACKAGES)]
  new_genomes <- c(new_genomes, names(.ORG_PACKAGES)[!names(.ORG_PACKAGES) %in% gsub("[0-9]+", "", c(new_genomes, genomes))])
  tab_new <- data.frame(new_genomes, "", "", FALSE, TRUE)
  colnames(tab_new) <- c(
    "Genome", "Id", "Id Description", "Lengths in geneLeneDataBase",
    "GO Annotation Available"
  )
  tab_supported <- rbind(tab_temp, tab_new)
  tab_supported[order(tab_supported$Genome), ]
}

#' goseq 
#'
#' Detects Gene Ontology and/or other user defined categories which are 
#' over/under represented in RNA-seq data.
#'
#' @importFrom mgcv gam smoothCon s mono.con pcls Predict.matrix
#' @importFrom graphics lines plot
#' @importFrom stats approx binomial dhyper median phyper runif
#' @importFrom utils data installed.packages tail
#' @importFrom AnnotationDbi as.list select
#' @importFrom GO.db GO.db
#' @importFrom GenomicFeatures transcriptLengths
#' @importFrom Seqinfo `genome<-`
#' @importFrom methods show
#' @importFrom rtracklayer browserSession ucscGenomes ucscTableQuery getTable
#' @importFrom BiocGenerics relist toTable
#' @importFrom BiasedUrn dWNCHypergeo pWNCHypergeo
#' @import geneLenDataBase
#'
#' @name goseq-pkg
#' @docType package
#' @keywords internal
"_PACKAGE"

#' Fetch GO categories
#' 
#' Obtains all gene ontology (GO) categories associated with a set of genes
#' using the relevant organism package.
#' 
#' This function attempts to make use of the organism packages
#' (org.<Genome>.<GeneID>.db) to obtain the mapping between gene ID and GO
#' categories.  As with \code{\link{getlength}} it is preferable that the same
#' gene identifier system is used for both summarization and retrieving GO
#' categories.
#' 
#' Valid options for the \code{fetch.cats} argument are any combination of
#' "GO:CC", "GO:BP", "GO:MF" & "KEGG".  The three GO terms refer to the
#' Cellular Component, Biological Process and Molecular Function respectively.
#' "KEGG" refers to KEGG pathways.
#' 
#' Note that \code{getgo} is a convenience function, designed to make
#' extracting mappings between GO categories and Gene ID easy.  For less common
#' organisms and/or gene ID \code{getgo} may fail to return a mapping even when
#' a legitimate mapping exists in the relevant organism package.  If
#' \code{getgo} fails, you should always try to build the mapping yourself from
#' the organism package (if one exists) before deciding that the information is
#' unavailable.  Further information and examples of this can be found in the
#' package Vignette.
#' 
#' @param genes A vector or list of genes to get the associated GO categories.
#' @param genome A string identifying the genome that \code{genes} refer to.
#' For a list of supported organisms run \code{\link{supportedGenomes}}.
#' @param id A string identifying the gene identifier used by \code{genes}.
#' For a list of supported gene IDs run \code{\link{supportedGeneIDs}}.
#' @param fetch.cats A vector specifying which categories to fetch the mapping
#' between category names and genes for.  See details for valid options.
#' @return A list where each entry is named by a gene and contains a vector of
#' all the associated GO categories.  This can be used directly with the
#' \code{gene2cat} option in \code{\link{goseq}}.
#' 
#' @author Matthew D. Young \email{myoung@@wehi.edu.au}
#' 
#' @seealso \code{\link{supportedGenomes}}, \code{\link{supportedGeneIDs}},
#' \code{\link{goseq}}
#' 
#' @export
#' 
#' @examples
#' 
#' genes <- c("ENSG00000124208", 
#'            "ENSG00000182463", 
#'            "ENSG00000124201", 
#'            "ENSG00000124205", 
#'            "ENSG00000124207")
#' getgo(genes,'hg19','ensGene')
#' 
getgo <- function(genes, genome, id, fetch.cats = c("GO:CC", "GO:BP", "GO:MF")) {
  # Check for valid input
  if (any(!fetch.cats %in% c("GO:CC", "GO:BP", "GO:MF", "KEGG"))) {
    stop("Invaled category specified.  Categories can only be GO:CC, GO:BP, GO:MF or KEGG")
  }
  # Convert from genome ID to org.__.__.db format
  orgstring <- as.character(.ORG_PACKAGES[match(gsub("[0-9]+", "", genome), names(.ORG_PACKAGES))])
  # Multimatch or no match
  if (length(orgstring) != 1) {
    stop("Couldn't grab GO categories automatically.  Please manually specify.")
  }
  # Load the library
  library(paste(orgstring, "db", sep = "."), character.only = TRUE)
  # What is the default ID that the organism package uses?
  coreid <- strsplit(orgstring, "\\.")[[1]][3]

  # Now we need to convert it into the naming convention used by the organism packages
  userid <- as.character(.ID_MAP[match(id, names(.ID_MAP))])
  # Multimatch or no match
  if (is.na(userid) | (length(userid) != 1)) {
    stop("Couldn't grab GO categories automatically.  Please manually specify.")
  }
  # The (now loaded) organism package contains a mapping between the internal ID and whatever
  # the default is (usually eg), the rest of this function is about changing that mapping to
  # point from categories to the ID specified
  # Fetch the mapping in its current format
  # Because GO is a directed graph, we need to get not just the genes associated with each ID,
  # but also those associated with its children.  GO2ALLEGS does this.
  core2cat <- NULL
  if (length(grep("^GO", fetch.cats)) != 0) {
    # Get the name of the function which maps gene ids to go terms
    # usually this will be "GO2ALLEG"
    gomapFunction <- .ORG_GOMAP_FUNCTION[orgstring]
    if (is.na(gomapFunction)) gomapFunction <- .ORG_GOMAP_FUNCTION["default"]
    x <- toTable(get(paste(orgstring, gomapFunction, sep = "")))
    # Keep only those ones that we specified and keep only the names
    # 		core2cat=x[x$Ontology%in%gsub("^GO:",'',fetch.cats),1:2]
    x[!x$Ontology %in% gsub("^GO:", "", fetch.cats), 2] <- "Other"
    core2cat <- x[, 1:2]
    colnames(core2cat) <- c("gene_id", "category")
  }
  if (length(grep("^KEGG", fetch.cats)) != 0) {
    x <- toTable(get(paste(orgstring, "PATH", sep = "")))
    # Either add it to existing table or create a new one
    colnames(x) <- c("gene_id", "category")
    if (!is.null(core2cat)) {
      core2cat <- rbind(core2cat, x)
    } else {
      core2cat <- x
    }
  }

  # Now we MAY have to convert the "gene_id" column to the format we are using
  if (coreid != userid) {
    # The mapping between user id and core id, don't use the <USER_ID>2<CORE_ID> object as the naming is not always consistent
    user2core <- toTable(get(paste(orgstring, userid, sep = "")))
    # Throw away any user ID that doesn't appear in core2cat
    user2core <- user2core[user2core[, 1] %in% core2cat[, 1], ]
    # Make a list version of core2cat, we'll need it
    list_core2cat <- split(core2cat[, 2], core2cat[, 1])
    # Now we need to replicate the core IDs that need replicating
    list_core2cat <- list_core2cat[match(user2core[, 1], names(list_core2cat))]
    # Now we can replace the labels on this list with the user ones from user2core,
    # but there will be duplicates, so we have to unlist, label, then relist
    user2cat <- split(unlist(list_core2cat, FALSE, FALSE), rep(user2core[, 2], sapply(list_core2cat, length)))
    # Now we only want each category listed once for each entry...
    user2cat <- sapply(user2cat, unique)
    ### In case you don't believe that this works as it should, here is the slow as all hell way for comparison...
    ### Make first list
    ## list_user2core=split(user2core[,1],user2core[,2])
    ### Make the second
    ## list_core2cat=split(core2cat[,2],core2cat[,1])
    ### Go through each entry in first list and expand using second...
    ## user2cat=sapply(list_user2core,function(u){unique(unlist(list_core2cat[u],FALSE,FALSE))})
  } else {
    # We don't need to convert anything (WOO!), so just make it into a list
    user2cat <- split(core2cat[, 2], core2cat[, 1])
    user2cat <- sapply(user2cat, unique)
  }
  # remove any empty strings
  user2cat <- lapply(user2cat, function(x) {
    if (length(x) > 1) x <- x[x != "Other"]
    x
  })

  ## we don't like case sensitivity
  names(user2cat) <- toupper(names(user2cat))

  # Now look them up
  return(user2cat[toupper(genes)])
}

#' goseq Gene Ontology analyser
#' 
#' Does selection-unbiased testing for category enrichment amongst
#' differentially expressed (DE) genes for RNA-seq data.  By default, tests
#' gene ontology (GO) categories, but any categories may be tested.
#' 
#' The \code{pwf} argument is almost always the output of the function
#' \code{\link{nullp}}.  This is a data frame with 3 columns, named "DEgenes",
#' "bias.data" and "pwf" with the rownames set to the gene names.  Each row
#' corresponds to a gene with the DEgenes column specifying if the gene is DE
#' (1 for DE, 0 for not DE), the bias.data column giving the numeric value of
#' the DE bias being accounted for (usually the gene length or number of
#' counts) and the pwf column giving the genes value on the probability
#' weighting function.
#' 
#' \code{goseq} obtains length data from UCSC and GO mappings from the organism
#' packages (see \code{link{getgo}} and \code{\link{getlength}} for details).
#' If your data is in an unsupported format you will need to obtain the GO
#' category mapping and supply them to the \code{goseq} function using the
#' \code{gene2cat} argument.
#' 
#' To use your own gene to category mapping with \code{goseq}, use the
#' \code{gene2cat} argument.  This argument takes a data.frame, with one column
#' containing gene IDs and the other containing the associated categories.  As
#' the mapping from gene <-> category is in general many to many there will be
#' multiple rows containing the same gene identifier.  Alternatively,
#' \code{gene2cat} can take a list, where the names are the genes and the
#' entries are the GO categories associated with the genes.  This is the format
#' produced by the \code{\link{getgo}} function and is more space efficient
#' than the data.frame representation.
#' 
#' If \code{gene2cat} is left as \code{NULL}, \code{goseq} attempts to use
#' \code{\link{getgo}} to fetch GO category to gene identifier mappings.
#' 
#' The PWF is usually calculated using the \code{\link{nullp}} function to
#' correct for length bias. However, \code{goseq} will work with any vector of
#' weights.  Any bias can be accounted for so long as a weight for each gene is
#' supplied using this argument.  \code{NA}s are allowed in the "pwf" and
#' "bias.data" columns of the PWF data frame (these usually occur as a result
#' of missing length data for some genes).  Any entry which is \code{NA} is set
#' to the weighting of the median gene.
#' 
#' Valid options for the \code{test.cats} argument are any combination of
#' "GO:CC", "GO:BP", "GO:MF" & "KEGG".  The three GO terms refer to the
#' Cellular Component, Biological Process and Molecular Function respectively.
#' "KEGG" refers to KEGG pathways.
#' 
#' The three methods, "Wallenius", "Sampling" & "Hypergeometric", calculate the
#' p-values as follows.
#' 
#' "Wallenius" approximates the true distribution of numbers of members of a
#' category amongst DE genes by the Wallenius non-central hypergeometric
#' distribution.  This distribution assumes that within a category all genes
#' have the same probability of being chosen.  Therefore, this approximation
#' works best when the range in probabilities obtained by the probability
#' weighting function is small. "Wallenius" is the recommended method for
#' calculating p-values.
#' 
#' "Sampling" uses random sampling to approximate the true distribution and
#' uses it to calculate the p-values for over (and under) representation of
#' categories. In practice, its use quickly becomes computationally prohibitive
#' because \code{repcnt} would need to be set very high for most applications.
#' 
#' CAUTION: "Hypergeometric" should NEVER be used for producing results for
#' biological interpretation.  If there is genuinely no bias in power to detect
#' DE in your experiment, the PWF will reflect this and the other methods will
#' produce accurate results.
#' 
#' "Hypergeometric" assumes there is no bias in power to detect differential
#' expression at all and calculates the p-values using a standard
#' hypergeometric distribution.  Useful if you wish to test the effect of
#' selection bias on your results.
#' 
#' @param pwf An object containing gene names, DE calls, the probability
#' weighting function. Usually generated by \code{\link{nullp}}.
#' @param genome A string identifying the genome that \code{genes} refer to.
#' For a list of supported organisms run \code{\link{supportedGenomes}}.
#' @param id A string identifying the gene identifier used by \code{genes}.
#' For a list of supported gene IDs run \code{\link{supportedGeneIDs}}.
#' @param gene2cat A data frame with two columns containing the mapping between
#' genes and the categories of interest.  Alternatively, a list where the names
#' are genes and each entry is a vector containing GO categories associated
#' with that gene (this is the output produced by \code{\link{getgo}}).  If set
#' to \code{NULL} goseq attempts to fetch GO categories automatically using
#' \code{\link{getgo}}.
#' @param test.cats A vector specifying which categories to test for over
#' representation amongst DE genes.  See details for allowed options.
#' @param method The method to use to calculate the unbiased category
#' enrichment scores.  Valid options are "Wallenius", "Sampling" &
#' "Hypergeometric".  "Hypergeometric" and "Sampling" should almost never be
#' used (see details).
#' @param repcnt Number of random samples to be calculated when random sampling
#' is used.  Ignored unless \code{method="Sampling"}.
#' @param use_genes_without_cat A boolean to indicate whether genes without a
#' category should still be used. For example, a large number of gene may have
#' no GO term annotated. If this option is set to FALSE, those genes will be
#' ignored in the calculation of p-values (default behaviour). If this option
#' is set to TRUE, then these genes will count towards the total number of
#' genes outside the category being tested (default behaviour prior to version
#' 1.15.2).
#' @return goseq returns a data frame with several columns.  The first column
#' gives the name of the category, the second gives the p-value for the
#' associated category being over represented amongst DE genes.  The third
#' column gives the p-value for the associated category being under represented
#' amongst DE genes.  The p-values have not been corrected for multiple
#' hypothesis testing. The fourth and fifth columns give the number of
#' differentially expressed genes in the category and total genes in the
#' category respectively. If any of the categories was a GO term, there will be
#' two additional columns for the GO term and its ontology.
#' 
#' @author Matthew D. Young \email{myoung@@wehi.edu.au}
#' 
#' @seealso \code{\link{nullp}}, \code{\link{getgo}}, \code{\link{getlength}}
#' 
#' @export
#' 
#' @references Young, M. D., Wakefield, M. J., Smyth, G. K., Oshlack, A. (2010)
#' \emph{Gene ontology analysis for RNA-seq: accounting for selection bias}
#' Genome Biology Date: Feb 2010 Vol: 11 Issue: 2 Pages: R14
#' 
#' @examples
#' 
#' data(genes)
#' pwf <- nullp(genes,'hg19','ensGene')
#' pvals <- goseq(pwf,'hg19','ensGene')
#' head(pvals)
#' 
goseq <- function(pwf, genome, id, gene2cat = NULL, test.cats = c("GO:CC", "GO:BP", "GO:MF"), method = "Wallenius", repcnt = 2000, use_genes_without_cat = FALSE) {
  ################# Input pre-processing and validation ###################
  # Do some validation of input variables
  if (any(!test.cats %in% c("GO:CC", "GO:BP", "GO:MF", "KEGG"))) {
    stop("Invalid category specified.  Valid categories are GO:CC, GO:BP, GO:MF or KEGG")
  }
  if ((missing(genome) | missing(id))) {
    if (is.null(gene2cat)) {
      stop("You must specify the genome and gene ID format when automatically fetching gene to GO category mappings.")
    }
    # If we're using user specified mappings, this obviously isn't a problem
    genome <- "dummy"
    id <- "dummy"
  }
  if (!any(method %in% c("Wallenius", "Sampling", "Hypergeometric"))) {
    stop("Invalid calculation method selected.  Valid options are Wallenius, Sampling & Hypergeometric.")
  }
  if (!is.null(gene2cat) && (!is.data.frame(gene2cat) & !is.list(gene2cat))) {
    stop("Was expecting a dataframe or a list mapping categories to genes.  Check gene2cat input and try again.")
  }

  # Factors are evil
  pwf <- unfactor(pwf)
  gene2cat <- unfactor(gene2cat)

  ###################### Data fetching and processing ########################
  if (is.null(gene2cat)) {
    # When we fetch the data using getgo it will be in the list format
    message("Fetching GO annotations...")
    gene2cat <- getgo(rownames(pwf), genome, id, fetch.cats = test.cats)
    names(gene2cat) <- rownames(pwf)
    # Do the two rebuilds to remove any nulls
    cat2gene <- reversemapping(gene2cat)
    gene2cat <- reversemapping(cat2gene)
  } else {
    # The gene2cat input accepts a number of formats, we need to check each of them in term
    message("Using manually entered categories.")
    # The options are a flat mapping (that is a data frame or matrix) or a list, where the list can be either gene->categories or category->genes
    if (class(gene2cat) != "list") {
      # it's not a list so it must be a data.frame, work out which column contains the genes
      genecol_sum <- as.numeric(apply(gene2cat, 2, function(u) {
        sum(u %in% rownames(pwf))
      }))
      genecol <- which(genecol_sum != 0)
      if (length(genecol) > 1) {
        genecol <- genecol[order(-genecol_sum)[1]]
        warning(paste("More than one possible gene column found in gene2cat, using the one headed", colnames(gene2cat)[genecol]))
      }
      if (length(genecol) == 0) {
        genecol <- 1
        warning(paste("Gene column could not be identified in gene2cat conclusively, using the one headed", colnames(gene2cat)[genecol]))
      }
      othercol <- 1
      if (genecol == 1) {
        othercol <- 2
      }
      # Now put it into our delicious listy format
      gene2cat <- split(gene2cat[, othercol], gene2cat[, genecol])
      # Do the appropriate builds
      cat2gene <- reversemapping(gene2cat)
      gene2cat <- reversemapping(cat2gene)
    }
    # !!!!
    # The following conditional has been flagged as a potential issue when using certain
    # types of input where the category names are the same as gene names (which seems like
    # something you should avoid anyway...).  Leave it for now
    # !!!!
    # We're now garunteed to have a list (unless the user screwed up the input) but it could
    # be category->genes rather than the gene->categories that we want.
    if (sum(unique(unlist(gene2cat, use.names = FALSE)) %in% rownames(pwf)) > sum(unique(names(gene2cat)) %in% rownames(pwf))) {
      gene2cat <- reversemapping(gene2cat)
    }
    # Alright, we're garunteed a list going in the direction we want now.  Throw out genes which we will not use
    gene2cat <- gene2cat[names(gene2cat) %in% rownames(pwf)]

    # Rebuild because it's a fun thing to do
    cat2gene <- reversemapping(gene2cat)
    gene2cat <- reversemapping(cat2gene)

    ## make sure we remove duplicate entries .. e.g. see
    ## http://permalink.gmane.org/gmane.science.biology.informatics.conductor/46876
    cat2gene <- lapply(cat2gene, function(x) {
      unique(x)
    })
    gene2cat <- lapply(gene2cat, function(x) {
      unique(x)
    })
  }

  nafrac <- (sum(is.na(pwf$pwf)) / nrow(pwf)) * 100
  if (nafrac > 50) {
    warning(paste("Missing length data for ", round(nafrac), "% of genes.  Accuracy of GO test will be reduced.", sep = ""))
  }
  # Give the genes with unknown length the weight used by the median gene (not the median weighting!)
  pwf$pwf[is.na(pwf$pwf)] <- pwf$pwf[match(sort(pwf$bias.data[!is.na(pwf$bias.data)])[ceiling(sum(!is.na(pwf$bias.data)) / 2)], pwf$bias.data)]

  ###################### Calculating the p-values ########################
  # Remove all the genes with unknown GOterms
  unknown_go_terms <- nrow(pwf) - length(gene2cat)
  if ((!use_genes_without_cat) && unknown_go_terms > 0) {
    message(paste("For", unknown_go_terms, "genes, we could not find any categories. These genes will be excluded."))
    message("To force their use, please run with use_genes_without_cat=TRUE (see documentation).")
    message("This was the default behavior for version 1.15.1 and earlier.")
    pwf <- pwf[rownames(pwf) %in% names(gene2cat), ]
  }
  # A few variables are always useful so calculate them
  cats <- names(cat2gene)
  DE <- rownames(pwf)[pwf$DEgenes == 1]
  num_de <- length(DE)
  num_genes <- nrow(pwf)
  pvals <- data.frame(category = cats, over_represented_pvalue = NA, under_represented_pvalue = NA, stringsAsFactors = FALSE, numDEInCat = NA, numInCat = NA)
  if (method == "Sampling") {
    # We need to know the number of DE genes in each category, make this as a mask that we can use later...
    num_DE_mask <- rep(0, length(cats))
    a <- table(unlist(gene2cat[DE], FALSE, FALSE))
    num_DE_mask[match(names(a), cats)] <- as.numeric(a)
    num_DE_mask <- as.integer(num_DE_mask)
    # We have to ensure that genes not associated with a category are included in the simulation, to do this they need an empty entry in the gene2cat list
    gene2cat <- gene2cat[rownames(pwf)]
    names(gene2cat) <- rownames(pwf)
    message("Running the simulation...")
    # Now do the actual simulating
    lookup <- matrix(0, nrow = repcnt, ncol = length(cats))
    for (i in 1:repcnt) {
      # A more efficient way of doing weighted random sampling without replacment than the built in function
      # The order(runif...)[1:n] bit picks n genes at random, weighting them by the PWF
      # The table(as.character(unlist(...))) bit then counts the number of times this random set occured in each category
      a <- table(as.character(unlist(gene2cat[order(runif(num_genes)^(1 / pwf$pwf), decreasing = TRUE)[1:num_de]], FALSE, FALSE)))
      lookup[i, match(names(a), cats)] <- a
      pp(repcnt)
    }
    message("Calculating the p-values...")
    # The only advantage of the loop is it uses less memory...
    # for(i in 1:length(cats)){
    # 	pvals[i,2:3]=c((sum(lookup[,i]>=num_DE_mask[i])+1)/(repcnt+1),(sum(lookup[,i]<=num_DE_mask[i])+1)/(repcnt+1))
    # 	pp(length(cats))
    # }
    pvals[, 2] <- (colSums(lookup >= outer(rep(1, repcnt), num_DE_mask)) + 1) / (repcnt + 1)
    pvals[, 3] <- (colSums(lookup <= outer(rep(1, repcnt), num_DE_mask)) + 1) / (repcnt + 1)
  }
  if (method == "Wallenius") {
    message("Calculating the p-values...")
    # All these things are just to make stuff run faster, mostly because comparison of integers is faster than string comparison
    degenesnum <- which(pwf$DEgenes == 1)
    # Turn all genes into a reference to the pwf object
    cat2genenum <- relist(match(unlist(cat2gene), rownames(pwf)), cat2gene)
    # This value is used in every calculation, by storing it we need only calculate it once
    alpha <- sum(pwf$pwf)
    # Each category will have a different weighting so needs its own test
    pvals[, 2:3] <- t(sapply(cat2genenum, function(u) {
      # The number of DE genes in this category
      num_de_incat <- sum(degenesnum %in% u)
      # The total number of genes in this category
      num_incat <- length(u)
      # This is just a quick way of calculating weight=avg(PWF within category)/avg(PWF outside of category)
      avg_weight <- mean(pwf$pwf[u])
      weight <- (avg_weight * (num_genes - num_incat)) / (alpha - num_incat * avg_weight)
      if (num_incat == num_genes) {
        weight <- 1
      } # case for the root GO terms
      # Now calculate the sum of the tails of the Wallenius distribution (the p-values)
      c(
        dWNCHypergeo(num_de_incat, num_incat, num_genes - num_incat, num_de, weight)
        + pWNCHypergeo(num_de_incat, num_incat, num_genes - num_incat, num_de, weight, lower.tail = FALSE),
        pWNCHypergeo(num_de_incat, num_incat, num_genes - num_incat, num_de, weight)
      )
    }))
  }
  if (method == "Hypergeometric") {
    message("Calculating the p-values...")
    # All these things are just to make stuff run faster, mostly because comparison of integers is faster than string comparison
    degenesnum <- which(pwf$DEgenes == 1)
    # Turn all genes into a reference to the pwf object
    cat2genenum <- relist(match(unlist(cat2gene), rownames(pwf)), cat2gene)
    # Simple hypergeometric test, one category at a time
    pvals[, 2:3] <- t(sapply(cat2genenum, function(u) {
      # The number of DE genes in this category
      num_de_incat <- sum(degenesnum %in% u)
      # The total number of genes in this category
      num_incat <- length(u)
      # Calculate the sum of the tails of the hypergeometric distribution (the p-values)
      c(dhyper(num_de_incat, num_incat, num_genes - num_incat, num_de) + phyper(num_de_incat, num_incat, num_genes - num_incat, num_de, lower.tail = FALSE), phyper(num_de_incat, num_incat, num_genes - num_incat, num_de))
    }))
  }
  # Populate the count columns...
  degenesnum <- which(pwf$DEgenes == 1)
  cat2genenum <- relist(match(unlist(cat2gene), rownames(pwf)), cat2gene)
  pvals[, 4:5] <- t(sapply(cat2genenum, function(u) {
    c(sum(degenesnum %in% u), length(u))
  }))

  # Finally, sort by p-value
  pvals <- pvals[order(pvals$over_represented_pvalue), ]

  # Supplement the table with the GO term name and ontology group
  # but only if the enrichment categories are actually GO terms
  if (any(grep("^GO:", pvals$category))) {
    GOnames <- select(GO.db, keys = pvals$category, columns = c("TERM", "ONTOLOGY"))[, 2:3]
    colnames(GOnames) <- tolower(colnames(GOnames))
    pvals <- cbind(pvals, GOnames)
  }

  # And return
  return(pvals)
}

#' Probability Weighting Function
#' 
#' Calculates a Probability Weighting Function for a set of genes based on a
#' given set of biased data (usually gene length) and each genes status as
#' differentially expressed or not.
#' 
#' It is essential that the entire analysis pipeline, from summarizing raw
#' reads through to using \code{goseq} be done in just one gene identifier
#' format.  If your data is in a different format you will need to obtain the
#' gene lengths and supply them to the \code{nullp} function using the
#' \code{bias.data} argument.  Converting to a supported format from another
#' format should be avoided whenever possible as this will almost always result
#' in data loss.
#' 
#' \code{NA}s are allowed in the bias.data vector if you do not have
#' information about a certain gene.  Setting a gene to \code{NA} is preferable
#' to removing it from the analysis.
#' 
#' If \code{bias.data} is left as \code{NULL}, \code{nullp} attempts to use
#' \code{\link{getlength}} to fetch GO category to gene identifier mappings.
#' 
#' It is recommended you review the fit produced by the \code{nullp} function
#' before proceeding by leaving \code{plot.fit} as \code{TRUE}.
#' 
#' @param DEgenes A named binary vector where 1 represents DE, 0 not DE and the
#' names are gene IDs.
#' @param genome A string identifying the genome that \code{genes} refer to.
#' For a list of supported organisms run \code{\link{supportedGenomes}}.
#' @param id A string identifying the gene identifier used by \code{genes}.
#' For a list of supported gene IDs run \code{\link{supportedGeneIDs}}.
#' @param bias.data A numeric vector containing the data on which the DE may
#' depend.  Usually this is the median transcript length of each gene in bp.
#' If set to \code{NULL} \code{nullp} will attempt to fetch length using
#' \code{\link{getlength}}.
#' @param plot.fit Plot the PWF or not?  Calls \code{\link{plotPWF}} with
#' default values if \code{TRUE}.
#' @return A data frame with 3 columns, named "DEgenes", "bias.data" and "pwf"
#' with the rownames set to the gene names.  Each row corresponds to a gene
#' with the DEgenes column specifying if the gene is DE (1 for DE, 0 for not
#' DE), the bias.data column giving the numeric value of the DE bias being
#' accounted for (usually the gene length or number of counts) and the pwf
#' column giving the genes value on the probability weighting function. This
#' object is usually passed to \code{goseq} to calculate enriched categories or
#' \code{plotPWF} for further plotting.
#' 
#' @author Matthew D. Young \email{myoung@@wehi.edu.au}
#' 
#' @seealso \code{\link{supportedGenomes}}, \code{\link{supportedGeneIDs}},
#' \code{\link{goseq}}, \code{\link{getlength}}
#' 
#' @export
#' 
#' @references Young, M. D., Wakefield, M. J., Smyth, G. K., Oshlack, A. (2010)
#' \emph{Gene ontology analysis for RNA-seq: accounting for selection bias}
#' Genome Biology Date: Feb 2010 Vol: 11 Issue: 2 Pages: R14
#' 
#' @examples
#' data(genes)
#' pwf <- nullp(genes, 'hg19', 'ensGene')
#' 
nullp <- function(DEgenes, genome, id, bias.data = NULL, plot.fit = TRUE) {
  # Input Checking
  if (!is.null(bias.data) & length(bias.data) != length(DEgenes)) {
    stop("bias.data vector must have the same length as DEgenes vector!")
  }
  # Factors cause strange things to happen, remove them if they exist
  bias.data <- unfactor(bias.data)
  DEgenes <- unfactor(DEgenes)

  # Fetch length data from geneLenDataBase
  if (is.null(bias.data)) {
    bias.data <- getlength(names(DEgenes), genome, id)
  }

  # Fit a spline to the binary vector of DE calls vs gene length
  # May not have bias data for some of the entries, return NA at those positions
  pwf <- rep(NA, length(DEgenes))
  w <- !is.na(bias.data)
  pwf[w] <- makespline(bias.data[w], DEgenes[w])

  # Make a data frame which contains all the data used to make the fit and the fit itself
  out <- data.frame(DEgenes = DEgenes, bias.data = bias.data, pwf = pwf, stringsAsFactors = FALSE)
  rownames(out) <- names(DEgenes)

  # Plot the PWF if the arument has been specified
  if (plot.fit) {
    plotPWF(out)
  }
  out
}

#' Monotonic Spline
#' 
#' Fits a monotonic cubic spline to the data provided, using the penalized
#' constrained least squares method from the \code{mgcv} package.
#' 
#' This uses the \code{pcls} function from the \pkg{mgcv} package to produce
#' the fit.  The monotonicity constraint is enforced using \code{mono.con} from
#' the same package. The \code{lower_bound} argument is only used on the rare
#' occasions when the fitting function becomes negative or arbitrarily close to
#' zero.  If this does occur \code{lower_bound} is added everywhere to ensure
#' that no one length is given essentially infinite weighting.
#' 
#' @param x The predictor variable.
#' @param y The response variable.  Must be the same length as \code{x}.
#' @param newX The points at which to return the value on the fitted spline.
#' If not specified \code{x} is used.
#' @param nKnots The number of knots to use in fitting the spline.
#' @param lower_bound The spline cannot drop below this value.
#' @return Returns a vector of values containing the value of the fit at each
#' point \code{newX}.
#' 
#' @author Matthew D. Young \email{myoung@@wehi.edu.au}.
#' 
#' @export
#' 
#' @references Package \pkg{mgcv}.  In particular this function is a
#' modification of an example given in the man page for \code{pcls}.
#' 
#' @examples
#' y <- c( rbinom(50,p=0.4,size=1), rbinom(50,p=0.6,size=1) )
#' x <- 1:100
#' plot(x,y)
#' p <- makespline(x,y)
#' lines(x,p)
#' 
makespline <- function(x, y, newX = NULL, nKnots = 6, lower_bound = 10^-3) {
  # Should not be used outside of goseq package.  Refer to the help pages for pcls in the mgcv package for more general
  # contstrained spline fitting.
  # We handle montonocily decreasing problems by reformulating them as monotonicly increasing by reflecting about "infinity"
  # Compare the first 10% to the last 10%
  ww <- order(x)
  size <- ceiling(length(y) / 10)
  low <- sum(y[ww][1:size])
  hi <- sum(y[ww][(length(y) - size):length(y)])
  # Is the trend decreasing
  if (hi <= low) {
    # Reform as a montonicly increasing fit by reflecting about infitinity
    reflectionFactor <- 10^10
    x <- reflectionFactor - x
  }
  # The number of knots to use when generating the spline
  nKnots <- round(nKnots)
  if (is.null(newX)) {
    newX <- x
  }
  # We need to force the fit to go through (0,0), mono.con constructs a requirment that the LOWEST x VALUE be greater than 0
  # so to force the fit through (0,0) we add a fake lowest x value of 0 by adding in a dummy point.
  # We use an inequality constraint rather than equality to only skew the fit enough that the spline is always >0
  # This won't work if we have a monotonically decreasing function (and we don't need to it), so check that
  if (hi > low) {
    x <- c(0, x)
    y <- c(0, y)
  }
  f.ug <- gam(y ~ s(x, k = nKnots, bs = "cr"), family = binomial())
  dat <- data.frame(x = x, y = y)
  sm <- smoothCon(s(x, k = nKnots, bs = "cr"), dat, knots = NULL)[[1]]
  if (length(sm$xp) < 6) {
    warning("Few than 6 nKnots were specified!\n Possible inaccurate fitting!\n")
  }
  Fmonocon <- mono.con(sm$xp, TRUE)
  # The lower and upper parameters here don't seem to build the right constraints 100% of the time, so add them manually
  # ND-6/14- removing the upper limit constraint because it wasn't satisfied by initial parameters.
  # Hope it doesn't break the code.
  Fmonocon$A <- rbind(Fmonocon$A, c(1, rep(0, ncol(Fmonocon$A) - 1))) # ,c(rep(0,ncol(F$A)-1)),-1))
  Fmonocon$b <- c(Fmonocon$b, 0) # ,1) ND - should this last one be -1?
  G <- list(
    X = sm$X, C = matrix(0, 0, 0), sp = f.ug$sp, p = sm$xp,
    y = y, w = y * 0 + 1, Ain = Fmonocon$A, bin = Fmonocon$b, S = sm$S, off = 0
  )
  # do some error checking as pcls can return NaNs sometimes if this isn't true
  if (any((G$Ain %*% G$p - G$bin) < 0)) {
    show(G$p)
    stop("Constraints for spline fit not satisfied by initial parameters")
  }
  # Do the actual fitting with penalized contstrained least squares
  p <- pcls(G)

  # Do some error checking for the rare case that p returns "Na"
  if (any(is.na(p))) {
    warning("The splines fit failed! Returning a flat weight distribution. This may compromise your GO analysis, please check what happened.")
    return(rep(mean(y), length(newX)))
  }

  # Now what we want is the value of the spline at each data point x,
  fv <- Predict.matrix(sm, data.frame(x = newX)) %*% p
  fv <- as.vector(fv)

  # If pcls still fails for some reason and returns negative values (or values that are so low that they'll have an effective relative weight of Inf
  # then we need to add a little bit to every weight to make things non-negative, the choice of minimum weight is somewhat arbitrary and serves only to
  # ensure that we don't have positive weights that will be infinitely prefered as a ratio.
  if (min(fv) < lower_bound) {
    fv <- fv - min(fv) + lower_bound
  }
  return(fv)
}

#' Retrieves Gene length data
#' 
#' Gets the length of each gene in a vector.
#' 
#' Length data is obtained from data obtained from the UCSC genome browser for
#' each combination of \code{genome} and \code{id}.  As fetching this data at
#' runtime is time consuming, a local copy of the length information for common
#' genomes and gene ID are included in the \pkg{geneLenDataBase} package.  This
#' function uses this package to fetch the required data.
#' 
#' The length of a gene is taken to be the median length of all its mature,
#' mRNA, transcripts.  It is always preferable to obtain length information
#' directly for the gene ID used to summarize your count data, rather than
#' converting IDs and then using the supplied databases.  Even when two genes
#' have a one-to-one mapping between different identifier conventions (which is
#' often not the case), they frequently refer to slightly different regions of
#' the genome with different lengths.  It is therefore recommended that the
#' user perform the full analysis in terms of only one gene ID, or manually
#' obtain their own length data for the identifier used to bin reads by gene.
#' 
#' @param genes A vector or list of the genes for which length information is
#' required.
#' @param genome A string identifying the genome that \code{genes} refer to.
#' For a list of supported organisms run \code{\link{supportedGenomes}}.
#' @param id A string identifying the gene identifier used by \code{genes}.
#' For a list of supported gene IDs run \code{\link{supportedGeneIDs}}.
#' @return Returns a vector of the gene lengths, in the same order as
#' \code{genes}.  If length data is unavailable for a particular gene NA is
#' returned in that position.  The returned vector is intended for use with the
#' \code{bias.data} option of the \code{\link{nullp}} function.
#' 
#' @author Matthew D. Young \email{myoung@@wehi.edu.au}
#' 
#' @seealso \code{\link{supportedGenomes}}, \code{\link{supportedGeneIDs}},
#' \code{\link{nullp}}, \pkg{geneLenDataBase}
#' 
#' @export
#' 
#' @examples
#' 
#' genes <- c("ENSG00000124208", 
#'            "ENSG00000182463", 
#'            "ENSG00000124201", 
#'            "ENSG00000124205", 
#'            "ENSG00000124207")
#' getlength(genes,'hg19','ensGene')
#' 
getlength <- function(genes, genome, id) {
  # geneLenDataBase contains all the gene length data, check it is installed
  if (!any(library()$results[, "Package"] == "geneLenDataBase")) {
    stop("Package geneLenDataBase is required for automatic gene length lookup.")
  }
  # Get a list of the length data files that are in the package
  repo <- grep(".*\\..*\\.LENGTH", as.data.frame(data(package = "geneLenDataBase")$results,
    stringsAsFactors = FALSE
  )$Item, ignore.case = TRUE, value = TRUE)
  # Is this genome/id in the local repository?
  if (paste(genome, id, "LENGTH", sep = ".") %in% repo) {
    # Yes it is, so load it and group transcript lengths into genes
    message("Loading ", genome, " length data...")
    data(list = paste(genome, id, "LENGTH", sep = "."), package = "geneLenDataBase")
    data <- split(get(paste(genome, id, "LENGTH", sep = "."))$Length, get(paste(genome, id, "LENGTH", sep = "."))$Gene)

    # Get the names of the genes used in the genome package
    gene_names <- get(paste(genome, id, "LENGTH", sep = "."))$Gene
  } else { ### Try to find the gene lengths from TXDB annotation packages
    message("Can't find ", genome, "/", id, " length data in genLenDataBase...")

    ## Is there a TxDB package installed for the organism?
    installedPackages <- rownames(installed.packages())

    # first try looking for a match with the same type of gene IDs
    txdbPattern <- paste("TxDb", "*", genome, id, sep = ".")
    txdbPack <- installedPackages[grep(txdbPattern, installedPackages)]

    # if not look for the package with generic gene IDs and convert later
    if (length(txdbPack) == 0) {
      orgstring <- as.character(.ORG_PACKAGES[match(gsub("[0-9]+", "", genome), names(.ORG_PACKAGES))])
      packageExtension <- tail(strsplit(orgstring, ".", fixed = TRUE)[[1]], n = 1)
      genericID <- names(.ID_MAP[which(.ID_MAP == packageExtension)])
      txdbPattern <- paste("TxDb", "*", genome, genericID, sep = ".")
      txdbPack <- installedPackages[grep(txdbPattern, installedPackages)]
      if (length(txdbPack) > 0) {
        library(paste(orgstring, "db", sep = "."), character.only = TRUE)
        userid <- as.character(.ID_MAP[match(id, names(.ID_MAP))])
        user2core <- toTable(get(paste(orgstring, userid, sep = "")))
      }
    }
    if (length(txdbPack) > 0) {
      message("Found the annotation package, ", txdbPack)
      message("Trying to get the gene lengths from it.")
      library(txdbPack, character.only = TRUE)
      txdb <- get(txdbPack)
      txlens <- transcriptLengths(txdb)
      core2len <- split(txlens$tx_len, txlens$gene_id)
      if (length(grep(id, txdbPattern)) != 0) {
        data <- core2len
        gene_names <- names(data)
      } else { # We need to conver between gene IDs. Just use Matt's code from getgo.R
        # Now we need to replicate the core IDs that need replicating
        core2len <- core2len[match(user2core[, 1], names(core2len))]
        # Now we can replace the labels on this list with the user ones from user2core,
        # but there will be duplicates, so we have to unlist, label, then relist
        data <- split(unlist(core2len, FALSE, FALSE), rep(user2core[, 2], sapply(core2len, length)))
        gene_names <- names(data)
      }
    } else { ## Can't use TxDB
      if (length(txdbPack) == 0 & any(.TXDB_ORGS == genome)) {
        message("A TxDb annotation package exists for ", genome, ". Consider installing it to get the gene lengths.", sep = "")
      }
      # If it's not in geneLenDataBase, try to load the data from UCSC using rtracklayer
      if (!requireNamespace("rtracklayer")) {
        stop("Can't load the rtracklayer library. Is it installed?...")
      }
      message(
        "Trying to download from UCSC. ",
        "This might take a couple of minutes. "
      )

      tryCatch(
        {
          # If not, then the user will have to supply it themselves
          # download the relevant table from UCSC
          session <- browserSession()
          genome(session) <- genome
          if (id == "geneSymbol") { # take the geneSymbols from refSeq genes
            query <- ucscTableQuery(session, "refGene")
          } else {
            query <- ucscTableQuery(session, id)
          }
          transTable <- getTable(query)

          # all this below is to get the length information of each gene
          starts <- strsplit(as.character(transTable$exonStart), ",")
          ends <- strsplit(as.character(transTable$exonEnd), ",")
          exons <- mapply(data.frame, starts, ends, SIMPLIFY = FALSE)
          get_length <- function(x) {
            sum(as.integer(as.character(x[, 2])) - as.integer(as.character(x[, 1])) + 1)
          }
          lengths <- sapply(exons, get_length)

          # now we need to get the correct list of gene names.
          names2 <- as.character(transTable$name2)
          gene_names <- names2
          if (!any(toupper(genes) %in% toupper(names2))) {
            # check which column has the gene names
            names1 <- as.character(transTable$name1)
            if (any(toupper(genes) %in% toupper(names1))) {
              gene_names <- names1
            }
          }
          data <- split(lengths, gene_names)
        },
        error = function(e) {
          stop(
            "Length information for genome ", genome, " and gene ID ", id,
            " is not available. You will have to specify bias.data manually."
          )
        }
      )
    }
  }
  # In order to assign just one length to each gene, we store the Median, Min Max
  # and count of transcripts for each gene for later use
  len <- data.frame(
    Gene = names(data), Median = sapply(data, median), Min = sapply(data, min),
    Max = sapply(data, max), Count = sapply(data, length)
  )

  # How many of the genes match the names the user specified with "genes"
  matched_frac <- sum(toupper(genes) %in% toupper(gene_names)) / length(genes)
  # None? Throw an error.
  if (matched_frac == 0) {
    stop(
      "The gene names specified do not match the gene names for genome ",
      genome, " and ID ", id, ".\n\tGene names given were: ", paste(genes[1:10], collapse = ", "),
      "\n\tRequired gene names are: ", paste(gene_names[1:10], collapse = ", ")
    )
  }
  # Less than 60%? Give a warning and continue
  if (matched_frac < .6) {
    warning(
      "More than 40% of gene names specified did not match the gene names for genome ",
      genome, " and ID ", id, ".  No length data will be available for these genes.\n\tGene names which failed to match were: ",
      paste(genes[!genes %in% gene_names][1:10], collapse = ", "), "\n\tRequired gene names are: ",
      paste(gene_names[1:10], collapse = ", ")
    )
  }
  # Finally, return the median transcript length for each matching gene
  return(len$Median[match(toupper(genes), toupper(len$Gene))])
}

#' Plot the Probability Weighting Function
#' 
#' Plots the Probability Weighting Function created by \code{\link{nullp}} by
#' binning together genes.
#' 
#' This function is almost always called using the output from the
#' \code{\link{nullp}} function.  However, it can be used to visualize the
#' length (or any other type of quantifiable) bias in ability to detect DE in a
#' data set.  The \code{pwf} argument needs to be a data frame with 3 columns
#' each containing numeric entries (although NAs are permitted in the bias.data
#' and pwf columns), which must be named "DEgenes", "bias.data" and "pwf",
#' although they can appear in any order.  The row names are taken to be the
#' gene names. The DEgenes column should be 0s or 1s where 1 represents a DE
#' gene, 0 a gene which is not DE.  The bias.data column is a quantification of
#' the quantity for which there is a bias in detecting DE for the associated
#' gene, this is usually gene length or the number of counts associated with a
#' gene.  Finally, the pwf column gives the probability weighting to be applied
#' for a given gene.
#' 
#' @param pwf A data frame with 3 columns named DEgenes, bias.data & pwf and
#' row names giving the gene names.  Usually generated by \code{\link{nullp}}.
#' @param binsize Calculate and plot the fraction of genes that are DE in bins
#' of this size.  If set to "auto" the best binsize for visualization is
#' attempted to be found automatically.
#' @param pwf_col The colour of the probability weighting function
#' @param pwf_lwd The width of the probability weighting function
#' @param xlab The x-axis label. \code{<binsize>} is replaced by the binsize
#' used.
#' @param ylab The y-axis label.
#' @param ...  Extra arguments that are passed to plot.
#' @return Nothing is returned.
#' 
#' @author Matthew D. Young \email{myoung@@wehi.edu.au}
#' 
#' @seealso \code{\link{nullp}}
#' 
#' @export
#' 
#' @references Young, M. D., Wakefield, M. J., Smyth, G. K., Oshlack, A. (2010)
#' \emph{Gene ontology analysis for RNA-seq: accounting for selection bias}
#' Genome Biology Date: Feb 2010 Vol: 11 Issue: 2 Pages: R14
#' 
#' @examples
#' data(genes)
#' pwf <- nullp(genes, 'hg19', 'ensGene',plot.fit=FALSE)
#' plotPWF(pwf,binsize=200)
#' 
plotPWF <- function(pwf, binsize = "auto", pwf_col = 3, pwf_lwd = 2, xlab = "Biased Data in <binsize> gene bins.", ylab = "Proportion DE", ...) {
  # We shouldn't try and plot NAs obviously...
  w <- !is.na(pwf$bias.data)
  o <- order(pwf$bias.data[w])
  # What is the total range in the fit?
  rang <- max(pwf$pwf, na.rm = TRUE) - min(pwf$pwf, na.rm = TRUE)
  if (rang == 0 & binsize == "auto") binsize <- 1000
  if (binsize == "auto") {
    # A low number of starting genes to bin, usually 100
    binsize <- max(1, min(100, floor(sum(w) * .08)))
    resid <- rang
    # Turn off warnings till we've worked out what we're doing
    oldwarn <- options()$warn
    options(warn = -1)
    # Keep increasing the number of genes in each bin until the scatter around the lines reaches the cutoff.
    # Stop if we reach only 10 bins for the entire plot
    while (binsize <= floor(sum(w) * .1) & resid / rang > .001) {
      binsize <- binsize + 100
      # Assign each gene a "bin number"
      splitter <- ceiling(1:length(pwf$DEgenes[w][o]) / binsize)
      # Determine the percentage DE in each bin
      de <- sapply(split(pwf$DEgenes[w][o], splitter), mean)
      # Determine the average length in each bin
      binlen <- sapply(split(as.numeric(pwf$bias.data[w][o]), splitter), mean)
      # Calculate the residuals, how much the binned data deviates from the PWF
      resid <- sum((de - approx(pwf$bias.data[w][o], pwf$pwf[w][o], binlen)$y)^2) / length(binlen)
    }
    options(warn = oldwarn)
  } else {
    # Assign each gene a "bin number"
    splitter <- ceiling(1:length(pwf$DEgenes[w][o]) / binsize)
    # Determine the percentage DE in each bin
    de <- sapply(split(pwf$DEgenes[w][o], splitter), mean)
    # Determine the average length in each bin
    binlen <- sapply(split(as.numeric(pwf$bias.data[w][o]), splitter), mean)
  }
  # Now we've settled on a binsize, plot it
  # Did the user specify the labels? If so we can't put in the defaults or they'll be used twice and errors result.
  xlab <- gsub("<binsize>", as.character(binsize), xlab)
  if ("xlab" %in% names(list(...))) {
    if ("ylab" %in% names(list(...))) {
      plot(binlen, de, ...)
    } else {
      plot(binlen, de, ylab = ylab, ...)
    }
  } else if ("ylab" %in% names(list(...))) {
    plot(binlen, de, xlab = xlab, ...)
  } else {
    plot(binlen, de, xlab = xlab, ylab = ylab, ...)
  }
  # Add the PWF
  lines(pwf$bias.data[w][o], pwf$pwf[w][o], col = pwf_col, lwd = pwf_lwd)
}

\documentclass[12pt]{article}

\topmargin 0in
\headheight 0in
\headsep 0in
\oddsidemargin 0in
\evensidemargin 0in
\textwidth 176mm
\textheight 215mm
\usepackage{natbib}
\usepackage{Sweave}
\usepackage{url}
\DeclareGraphicsRule{.tif}{png}{.png}{`convert #1 `dirname #1`/`basename #1 .tif`.png}
\newcommand{\np}{\vspace{5mm}\par}

\begin{document}

%\VignetteIndexEntry{goseq User's Guide}

\title{\texttt{goseq}: Gene Ontology testing for RNA-seq datasets}
\author{Matthew D. Young
\and
Nadia Davidson % \\ \texttt{nadia.davidson@mcri.edu.au}
\and
Matthew J. Wakefield
\and
Gordon K.\ Smyth
\and
Alicia Oshlack}
% Please increment date when working on this document, so that
% date shows genuine change date, not merely date of compile.
\date{8 September 2017}
\maketitle

\section{Introduction}

This document gives an introduction to the use of the \texttt{goseq} R Bioconductor package~\citep{Young10}.  This package provides methods for performing Gene Ontology analysis of RNA-seq data, taking length bias into account~\citep{Oshlack:2009p149}.  The methods and software used by \texttt{goseq} are equally applicable to other category based test of RNA-seq data, such as KEGG pathway analysis.
\np
Once installed, the \texttt{goseq} package can be easily loaded into R using:
<<load_library>>=
library(goseq)
@
<<set_width,echo=FALSE>>=
options(width = 84)
@
\np
In order to perform a GO analysis of your RNA-seq data, \texttt{goseq} only requires a simple named vector, which contains two pieces of information.

\begin{enumerate}
\item  \texttt{Measured genes}: all genes for which RNA-seq data was gathered for your experiment.  Each element of your vector should be named by a unique gene identifier.
\item \texttt{Differentially expressed genes}:  each element of your vector should be either a 1 or a 0, where 1 indicates that the gene is differentially expressed and 0 that it is not.
\end{enumerate}

If the organism, gene identifier or category test is currently not natively supported by \texttt{goseq}, it will also be necessary to supply additional information regarding the genes length and/or the association between categories and genes.
\np
Bioconductor R packages such as \texttt{Rsubread} allow for the summarization of mapped reads into a table of counts, such as reads per gene.  From there, several packages exist for performing differential expression analysis on summarized data (eg. \texttt{edgeR}~\citep{Robinson07, Robinson08, Robinson10}).  \texttt{goseq} will work with any method for determining differential expression and as such differential expression analysis is outside the scope of this document, but in order to facilitate ease of use, we will make use of the \texttt{edgeR} package to calculate differentially expressed (DE) genes in all the case studies in this document.

\section{Reading data}

We assume that the user can use appropriate in-built \texttt{R} functions (such as \texttt{read.table} or \texttt{scan}) to obtain two vectors, one containing all genes assayed in the RNA-seq experiment, the other containing all genes which are DE.  If we assume that the vector of genes being assayed is named \texttt{assayed.genes} and the vector of DE genes is named \texttt{de.genes} we can construct a named vector suitable for use with \texttt{goseq} using the following:

<<read.data,eval=FALSE>>=
gene.vector <- as.integer(assayed.genes %in% de.genes)
names(gene.vector) <- assayed.genes
head(gene.vector)
@

It may be that the user can already read in a vector in this format, in which case it can then be immediately used by \texttt{goseq}.

\section{GO testing of RNA-seq data}

To begin the analysis, \texttt{goseq} first needs to quantify the length bias present in the dataset under consideration.  This is done by calculating a Probability Weighting Function or PWF which can be thought of as a function which gives the probability that a gene will be differentially expressed (DE), based on its length alone.  The PWF is calculated by fitting a monotonic spline to the binary data series of differential expression (1=DE, 0=Not DE) as a function of gene length.  The PWF is used to weight the chance of selecting each gene when forming a null distribution for GO category membership.  The fact that the PWF is calculated directly from the dataset under consideration makes this approach robust, only correcting for the length bias present in the data.  For example, if \texttt{goseq} is run on a microarray dataset, for which no length bias exists, the calculated PWF will be nearly flat and all genes will be weighted equally, resulting in no length bias correction.
\np
In order to account for the length bias inherent to RNA-seq data when performing a GO analysis (or other category based tests), one cannot simply use the hypergeometric distribution as the null distribution for category membership, which is appropriate for data without DE length bias, such as microarray data.  GO analysis of RNA-seq data requires the use of random sampling in order to generate a suitable null distribution for GO category membership and calculate each categories significance for over representation amongst DE genes.
\np
However, this random sampling is computationally expensive.  In most cases, the Wallenius distribution can be used to approximate the true null distribution, without any significant loss in accuracy.  The \texttt{goseq} package implements this approximation as its default option.  The option to generate the null distribution using random sampling is also included as an option, but users should be aware that the default number of samples generated will not be enough to accurately call enrichment when there are a large number of go terms.
\np
Having established a null distribution, each GO category is then tested for over and under representation amongst the set of differentially expressed genes and the null is used to calculate a p-value for under and over representation.

\section{Natively supported Gene Identifiers and category tests}

\texttt{goseq} needs to know the length of each gene, as well as what GO categories (or other categories of interest) each gene is associated with. \texttt{goseq} relies on the UCSC genome browser to provide the length information for each gene.  However, because the process of fetching the length of every transcript is slow and bandwidth intensive, \texttt{goseq} relies on an offline copy of this information stored in the data package \texttt{geneLenDataBase}.  To see which genome/gene identifier combinations are in the local database, simply run:
<<supported_genomes,eval=FALSE>>=
supportedOrganisms()
@
The leftmost columns in the output of this command list the genomes and gene identifiers respectively. If length data exists in the local database it is indicated in the second last column. If your genome/ID combination is not in the local database, it may be downloaded from the UCSC genome browser or taken from a \texttt{TxDb} annotation package (if installed). If your genome/ID combination is not found in any database, you will have to manually specify the gene lengths. We encourage all users to manually specify their gene lengths if provided by upstream summarization programs. e.g. \texttt{featureCounts}, as these lengths will be more accurate.
\np
In order to link GO categories to genes, \texttt{goseq} uses the organism packages from Bioconductor.  These packages are named org.<Genome>.<ID>.db, where <Genome> is a short string identifying the genome and <ID> is a short string identifying the gene identifier.  Currently, \texttt{goseq} will automatically retrieve the mapping between GO categories and genes from the relevant package (as long as it is installed) for commonly used genome/ID combinations.  If GO mappings are not automatically available for your genome/ID combination, you will have to manually specify the relationship between genes and categories.  Although the Genome/ID naming conventions used by the organism packages differ from the UCSC, \texttt{goseq} is able to convert between the two, so the user need only ever specify the UCSC genome/ID in most cases. The final column indicates whether the Genome/ID combination is supported for GO categories.

\section{Non-native Gene Identifier or category test}

If the organism, Gene Identifier or category test you wish to perform is not in the native \texttt{goseq} database, you will have to supply one or all of the following:

\begin{itemize}
	\item \texttt{Length data}: the length of each gene in your gene identifier format.
	\item \texttt{Category mappings}: the mapping (usually many-to-many) between the categories you wish to test for over/under representation amongst DE genes and genes in your gene identifier format.
\end{itemize}

\subsection{Length data format}

The length data must be formatted as a numeric vector, of the same length as the main named vector specifying gene names/DE genes.  Each entry should give the length of the corresponding gene in bp.  If length data is unavailable for some genes, that entry should be set to NA. 

\subsection{Category mapping format}

The mapping between category names and genes should be given as a data frame with two columns.  One column should contain the gene IDs and the other the name of an associated category.  As the mapping between categories and genes is usually many-to-many, this data frame will usually have multiple rows with the same gene name and category name.
\np
Alternatively, mappings between genes and categories can be given as a list.  The names of list entries should be gene IDs and the entries themselves should be a vector of category names to which the gene ID corresponds.

\subsection{Some additional tips}

Any organism for which there is an annotation on either Ensembl or the UCSC, can be easily turned into length data using the GenomicFeatures package.  To do this, first create a TranscriptDb object using either makeTxDbFromBiomart or makeTxDbFromUCSC (see the help in the GenomicFeatures package on using these commands).  Once you have a transcriptDb object, you can get a vector named by gene ID containing the median transcript length of each gene simply by using the command.

<<getLengthDataFromUCSC,eval=FALSE>>=
txsByGene <- transcriptsBy(txdb, "gene")
lengthData <- median(width(txsByGene))
@

The relationship between gene identifier and GO category can usually be obtained from the Gene Ontology website (www.geneontology.org) or from the NCBI.  Additionally, the bioconductor AnnotationDbi library has recently added a function "makeOrgPackageFromNCBI", which can be used to create an organism package from within R, using the NCBI data.  Once created, this package can then be used to obtain the mapping between genes and gene ontology.

\section{Case study: Prostate cancer data}
\subsection{Introduction}

This section provides an analysis of data from an RNA-seq experiment to illustrate the use of \texttt{goseq} for GO analysis.

\np
This experiment examined the effects of androgen stimulation on a human prostate cancer cell line, LNCaP. The data set includes more than 17 million short cDNA reads obtained for both the treated and untreated cell line and sequenced on Illumina's 1G genome analyzer. 

\np
For each sample we were provided with the raw 35 bp RNA-seq reads from the authors.  For the untreated prostate cancer cells (LNCaP cell line) there were 4 lanes totaling ~10 million, 35 bp reads.  For the treated cells there were 3 lanes totaling ~7 million, 35 bp reads.  All replicates were technical replicates.  Reads were mapped to NCBI version 36.3 of the human genome using bowtie.  Any read with which mapped to multiple locations was discarded.  Using the ENSEMBL 54 annotation from biomart, each mapped read was associated with an ENSEMBL gene.  This was done by associating any read that overlapped with any part of the gene (not just the exons) with that gene.  Reads that did not correspond to genes were discarded. 

\subsection{Source of the data}

The data set used in this case study is taken from \citep{Li:2008p113} and was made available from the authors upon request.  

\subsection{Determining the DE genes using \texttt{edgeR}}

To begin with, we load in the text data and convert it the appropriate \texttt{edgeR} DGEList object.

<<edger_1>>=
library(edgeR)
table.summary <- read.table(system.file("extdata", "Li_sum.txt", package = "goseq"),
  sep = "\t", header = TRUE, stringsAsFactors = FALSE
)
counts <- table.summary[, -1]
rownames(counts) <- table.summary[, 1]
grp <- factor(rep(c("Control", "Treated"), times = c(4, 3)))
summarized <- DGEList(counts, lib.size = colSums(counts), group = grp)
@

Next, we use \texttt{edgeR} to estimate the biological dispersion and calculate differential expression using a negative binomial model.

<<edger_2>>=
disp <- estimateCommonDisp(summarized)
disp$common.dispersion
tested <- exactTest(disp)
topTags(tested)
@

Finally, we Format the DE genes into a vector suitable for use with \texttt{goseq}

<<edger_3>>=
genes <- as.integer(p.adjust(tested$table$PValue[tested$table$logFC != 0],
  method = "BH"
) < .05)
names(genes) <- row.names(tested$table[tested$table$logFC != 0, ])
table(genes)
@

\subsection{Determining Genome and Gene ID}

In order to allow for automatic data retrieval, the user has to tell \texttt{goseq} what genome and gene ID format were used to summarize the data.  In our case we will use the hg19 build of the human genome, we check what code this corresponds to by running:

<<head_organisms,eval=TRUE>>=
head(supportedOrganisms())
@

Which lists the genome codes in the far left column, headed ``Genome''.  As we are using ``hg19" and  we also know that we used ENSEMBL Gene ID to summarize our read data, we check what code this corresponds to by running:

<<head_geneids>>=
supportedOrganisms()[supportedOrganisms()$Genome == "hg19", ]
@

The gene ID codes are listed in the column second from left, titled ``Id''.  We find that our gene ID code is ``ensGene".  We will use these strings whenever we are asked for a genome or id. If the gene ID is missing for your Genome (for example this is the case for hg38), then the genome is not supported in the geneLengthDatabase. Gene lengths will either be automatically fetched from TxDB, UCSC or you will need to provide them manually. Supported Gene IDs to automatically fetch GO terms should usually either be Entrez (``knownGene"), Ensembl (``ensGene") or gene symbols (``geneSymbol").

\subsection{GO analysis}

\subsubsection{Fitting the Probability Weighting Function (PWF)}

We first need to obtain a weighting for each gene, depending on its length, given by the PWF.  As you may have noticed when running supportedGenomes or supportedGeneIDs, length data is available in the local database for our gene ID, ``ensGene" and our genome, ``hg19".  We will let \texttt{goseq} automatically fetch this data from its databases.

<<pwf,fig=TRUE>>=
pwf <- nullp(genes, "hg19", "ensGene")
head(pwf)
@

\texttt{nullp} plots the resulting fit, allowing verification of the goodness of fit before continuing the analysis.  Further plotting of the pwf can be performed using the \texttt{plotPWF} function.
\np
The output of nullp contains all the data used to create the PWF, as well as the PWF itself.   It is a data frame with 3 columns, named "DEgenes", "bias.data" and "pwf" with the rownames set to the gene names.  Each row corresponds to a gene with the DEgenes column specifying if the gene is DE (1 for DE, 0 for not DE), the bias.data column giving the numeric value of the DE bias being accounted for (usually the gene length or number of counts) and the pwf column giving the genes value on the probability weighting function.


\subsubsection{Using the Wallenius approximation}

To start with we will use the default method, to calculate the over and under expressed GO categories among DE genes.  Again, we allow \texttt{goseq} to fetch data automatically, except this time the data being fetched is the relationship between ENSEMBL gene IDs and GO categories.

%This hides the output of goseq

<<GO.wall>>=
GO.wall <- goseq(pwf, "hg19", "ensGene")
head(GO.wall)
@

The resulting object is ordered by GO category over representation amongst DE genes.  

\subsubsection{Using random sampling}

It may sometimes be desirable to use random sampling to generate the null distribution for category membership. For example, to check consistency against results from the Wallenius approximation. This is easily accomplished by using the \texttt{method} option to specify sampling and the \texttt{repcnt} option to specify the number of samples to generate:

<<GO.samp,results=hide>>=
GO.samp <- goseq(pwf, "hg19", "ensGene", method = "Sampling", repcnt = 1000)
@
<<head_samp>>=
head(GO.samp)
@

You will notice that this takes far longer than the Wallenius approximation.  Plotting the p-values against one another, we see that there is little difference between the two methods. However, the accuracy of the sampling method is limited by the number of samples generated, \texttt{repcnt}, such that very low p-values will not be correctly calculated. Significantly enriched GO terms may then be missed after correcting for multiple testing.

<<plot_wal_v_samp,fig=TRUE>>=
plot(log10(GO.wall[, 2]), log10(GO.samp[match(GO.wall[, 1], GO.samp[, 1]), 2]),
  xlab = "log10(Wallenius p-values)", ylab = "log10(Sampling p-values)",
  xlim = c(-3, 0)
)
abline(0, 1, col = 3, lty = 2)
@

\subsubsection{Ignoring length bias}

\texttt{goseq} also allows for one to perform a GO analysis without correcting for RNA-seq length bias.  In practice, this is only useful for assessing the effect of length bias on your results.  You should NEVER use this option as your final analysis.  If length bias is truly not present in your data, \texttt{goseq} will produce a nearly flat PWF and no length bias correction will be applied to your data and all methods will produce the same results.
\np
However, if you still wish to ignore length bias in calculating GO category enrichment, this is again accomplished using the \texttt{method} option.

<<GO.nobias>>=
GO.nobias <- goseq(pwf, "hg19", "ensGene", method = "Hypergeometric")
head(GO.nobias)
@

Ignoring length bias gives very different results from a length bias corrected analysis.

<<plot_wal_v_hyper,fig=TRUE>>=
plot(log10(GO.wall[, 2]), log10(GO.nobias[match(GO.wall[, 1], GO.nobias[, 1]), 2]),
  xlab = "log10(Wallenius p-values)", ylab = "log10(Hypergeometric p-values)",
  xlim = c(-3, 0), ylim = c(-3, 0)
)
abline(0, 1, col = 3, lty = 2)
@

\subsubsection{Limiting GO categories and other category based tests}

By default, \texttt{goseq} tests all three major Gene Ontology branches; Cellular Components, Biological Processes and Molecular Functions.  However, it is possible to limit testing to any combination of the major branches by using the \texttt{test.cats} argument to the goseq function.  This is done by specifying a vector consisting of some combination of the strings ``GO:CC", ``GO:BP" and ``GO:MF".  For example, to test for only Molecular Function GO categories:

<<GO.limited>>=
GO.MF <- goseq(pwf, "hg19", "ensGene", test.cats = c("GO:MF"))
head(GO.MF)
@

Native support for other category tests, such as KEGG pathway analysis are also made available via this argument.  See the man \texttt{goseq} function man page for up to date information on what category tests are natively supported.

\subsubsection{Making sense of the results}

Having performed the GO analysis, you may now wish to interpret the results.  If you wish to identify categories significantly enriched/unenriched below some p-value cutoff, it is necessary to first apply some kind of multiple hypothesis testing correction.  For example, GO categories over enriched using a .05 FDR cutoff \citep{Benjamini95} are:

<<enriched_GO>>=
enriched.GO <- GO.wall$category[p.adjust(GO.wall$over_represented_pvalue,
  method = "BH"
) < .05]
head(enriched.GO)
@

Unless you are a machine, GO accession identifiers are probably not very meaningful to you.  Information about each term can be obtained from the Gene Ontology website, \url{http://www.geneontology.org/}, or using the R package \texttt{GO.db}.

<<GO_explained>>=
library(GO.db)
for (go in enriched.GO[1:10]) {
  print(GOTERM[[go]])
  cat("--------------------------------------\n")
}
@

\subsubsection{Understanding goseq internals}

The situation may arise where it is necessary for the user to perform some of the data processing steps usually performed automatically by \texttt{goseq} themselves.  With this in mind, it will be useful to step through the preprocessing steps performed automatically by \texttt{goseq} to understand what is happening.
\np
To start with, when \texttt{nullp} is called, \texttt{goseq} uses the genome and gene identifiers supplied to try and retrieve length information for all genes given to the \texttt{genes} argument.  To do this, it retrieves the data from the database of gene lengths maintained in the package \texttt{geneLenDataBase}.  This is performed by the \texttt{getlength} function in the following way:
<<getlength>>=
len <- getlength(names(genes), "hg19", "ensGene")
length(len)
length(genes)
head(len)
@
After some data cleanup, the length data and the DE data is then passed to the \texttt{makespline} function to produce the PWF.  The \texttt{nullp} returns a data frame which has 3 columns, the original DEgenes vector, the length bias data (in a column called bias.data) and the PWF itself (in a column named pwf).  The names of the genes are also kept in this data frame as the names of the rows. If length data could not be obtained for a certain gene the corresponding entries in the "bias.data" and "pwf" columns are set to NA.
\np
Next we call the \texttt{goseq} function to determine over/under representation of GO categories amongst DE genes.  When we do this, \texttt{goseq} looks for the appropriate organism package and tries to obtain the mapping from genes to GO categories from it.  This is done using the \texttt{getgo} function as follows:
<<getgo>>=
go <- getgo(names(genes), "hg19", "ensGene")
length(go)
length(genes)
head(go)
@

Note that some of the gene categories have been returned as "NULL". This means that a GO category could not be found in the database for 
one of the genes. In the \texttt{goseq} command, enrichment will only be calculated using genes with a GO category by default. However, in older 
versions of goseq (below 1.15.2), we counted all genes. i.e. genes with no categories still counted towards the total number of gene outside
of any single category. It is possible to switch between these two behaviors using the \texttt{use\_genes\_without\_cat} flag in \texttt{goseq}.


The first thing the getgo function does is to convert the UCSC genome/ID namings into the naming convention used by the organism packages.  This is done using two hard coded conversion vectors that are included in the \texttt{goseq} package but usually hidden from the user.

<<conv_table>>=
goseq:::.ID_MAP
goseq:::.ORG_PACKAGES
@

It is just as valid to run the length and GO category fetching as separate steps and then pass the result to the \texttt{nullp} and \texttt{goseq} functions using the \texttt{bias.data} and \texttt{gene2cat} arguments.  Thus the following two blocks of code are equivalent:

<<norm_analysis,eval=FALSE>>=
pwf <- nullp(genes, "hg19", "ensGene")
go <- goseq(pwf, "hg19", "ensGene")
@
and
<<verbose_analysis,eval=FALSE>>=
gene_lengths <- getlength(names(genes), "hg19", "ensGene")
pwf <- nullp(genes, bias.data = gene_lengths)
go_map <- getgo(names(genes), "hg19", "ensGene")
go <- goseq(pwf, "hg19", "ensGene", gene2cat = go_map)
@

\subsection{KEGG pathway analysis}

In order to illustrate performing a category test not present in the \texttt{goseq} database, we perform a KEGG pathway analysis.  For human, the mapping from KEGG pathways to genes are stored in the package org.Hs.eg.db, in the object org.Hs.egPATH.  In order to test for KEGG pathway over representation amongst DE genes, we need to extract this information and put it in a format that \texttt{goseq} understands.  Unfortunately, the org.Hs.eg.db package does not contain direct mappings between ENSEMBL gene ID and KEGG pathway.  Therefore, we have to construct this map by combining the ENSEMBL <-> Entrez and Entrez <-> KEGG mappings.  This can be done using the following code:

<<KEGG_mappings,eval=FALSE>>=
# Get the mapping from ENSEMBL 2 Entrez
en2eg <- as.list(org.Hs.egENSEMBL2EG)
# Get the mapping from Entrez 2 KEGG
eg2kegg <- as.list(org.Hs.egPATH)
# Define a function which gets all unique KEGG IDs
# associated with a set of Entrez IDs
grepKEGG <- function(id, mapkeys) {
  unique(unlist(mapkeys[id], use.names = FALSE))
}
# Apply this function to every entry in the mapping from
# ENSEMBL 2 Entrez to combine the two maps
kegg <- lapply(en2eg, grepKEGG, eg2kegg)
head(kegg)
@

Note that this step is quite time consuming.  The code written here is not the most efficient way of producing this result, but the logic is much clearer than faster algorithms.  The source code for getgo contains a more efficient routine.
\np
We produce the PWF as before.  Then, to perform a KEGG analysis, we simply make use of the \texttt{gene2cat} option in \texttt{goseq}:

<<KEGG,eval=FALSE>>=
pwf <- nullp(genes, "hg19", "ensGene")
KEGG <- goseq(pwf, gene2cat = kegg)
head(KEGG)
@

Note that we do not have to tell the goseq function what organism and gene ID we are using as we are manually supplying the mapping between genes and categories.
\np
KEGG analysis is shown as an illustration of how to supply your own mapping between gene ID and category, KEGG analysis is actually natively supported by GOseq and we could have performed it with the following code.

<<KEGG_goseq>>=
pwf <- nullp(genes, "hg19", "ensGene")
KEGG <- goseq(pwf, "hg19", "ensGene", test.cats = "KEGG")
head(KEGG)
@

Noting that this time it was necessary to tell the goseq function that we are using HG19 and ENSEMBL gene ID, as the function needs this information to automatically construct the mapping from geneid to KEGG pathway.

\subsection{Extracting mappings from organism packages}

If you know that the information mapping gene ID to your categories of interest is contained in the organism packages, but \texttt{goseq} fails to fetch it automatically, you may want to extract it yourself and then pass it to the \texttt{goseq} function using the \texttt{gene2cat} argument.  This is done in exactly the same way as extracting the KEGG to ENSEMBL mappings in the section ``KEGG pathway analysis" above.  This example is actually the worst case, where it is necessary to combine two mappings to get the desired list.  If we had instead wanted the association between Entrez gene IDs and KEGG pathways, the following code would have been sufficient:

<<KEGG_from_db>>=
kegg <- as.list(org.Hs.egPATH)
head(kegg)
@

A note on fetching GO mappings from the organism.  The data structure of GO is a directed acyclic graph.  This means that in addition to each GO category being associated with a set of genes, it may also have children that are associated to other genes.  It is important to use the org.Hs.egGO2ALLEGS and NOT the org.Hs.egGO object to create the mapping between GO categories and gene identifiers, as the latter does not include the links to genes arising from "child" GO categories.  Thank you to Christopher Fjell for pointing this out.

\subsection{Correcting for other biases}

It is possible that in some circumstances you will wish to correct not just for length bias, but for the total number of counts.  This can make sense because power to detect DE depends on the total number of counts a gene receives, which is the product of gene length and gene expression.  So correcting for read count bias will compensate for all biases, known and unknown, in power to detect DE.  On the other hand, it will also remove bias resulting from differences in expression level, which may not be desirable.
\np
Correcting for count bias will produce a different PWF.  Therefore, we need to tell \texttt{goseq} about the data on which the fraction DE depends when calculating the PWF using the \texttt{nullp} function.  We then simply pass the result to \texttt{goseq} as usual.
\np
So, in order to tell \texttt{goseq} to correct for read count bias instead of length bias, all you need to do is supply a numeric vector, containing the number of counts for each gene to \texttt{nullp}.

<<countbias>>=
countbias <- rowSums(counts)[rowSums(counts) != 0]
length(countbias)
length(genes)
@

To use the count bias when doing GO analysis, simply pass this vector to \texttt{nullp} using the \texttt{bias.data} option.  Note that we have to supply "hg19" and "ensGene" to \texttt{goseq} as it is not used by nullp and hence not in the pwf.counts object.

<<GO.counts>>=
pwf.counts <- nullp(genes, bias.data = countbias)
GO.counts <- goseq(pwf.counts, "hg19", "ensGene")
head(GO.counts)
@

Note that if you want to correct for length bias, but your organism/gene identifier is not natively supported, then you need to follow the same procedure as above, only the numeric vector supplied will contain each gene's length instead of its number of reads.
\section{Setup}

This vignette was built on:

<<setup>>=
sessionInfo()
@

\section{Acknowledgments}

Christopher Fjell for a series of bug fixes and pointing out the difference between the egGO and egGO2ALLEGS objects in the organism packages.

\bibliographystyle{plainnat}
\bibliography{goseq_bib}

\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/supportedOrganisms.R
\name{supportedOrganisms}
\alias{supportedOrganisms}
\title{Supported Organisms}
\usage{
supportedOrganisms()
}
\value{
A data.frame containing supported genomes and gene ids
}
\description{
Lists which genomes and gene ids are supported for gene lengths and for gene
ontology
}
\details{
Goseq allows a user to provide their own bias data (usually gene lengths)
and/or gene categories (usually gene ontologies), but goseq also provides
this data automatically for many commonly used species. This function lists
which genome and gene ids are automatically supported by goseq. The first to
third columns list the genomes, gene ids, and gene id descriptions
respectively. The fourth column indicates whether this combination of genome
and id are available in the geneLengthDataBase. If a particular combination
is absent, goseq may still automatically fetch the gene lengths from either
a TxDB annotation package (must be installed) or download the data from
UCSC. For example gene lengths for \code{hg38} are not supported in
\code{geneLengthDataBase} but may be fetched by these other means. However,
this is not always the case. The final column indicates if GO terms will be
automatically fetched for the genome and id combination.  Goseq relies on an
\code{org} annotation package (e.g. \code{org.Hs.eg.db}) existing for the
organism. In general, if either the lengths or GO terms are not supported,
the user must enter this information manually.
}
\examples{
supportedOrganisms()

}
\author{
Nadia Davidson \email{nadia.davidson@mcri.edu.au}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotPWF.R
\name{plotPWF}
\alias{plotPWF}
\title{Plot the Probability Weighting Function}
\usage{
plotPWF(
  pwf,
  binsize = "auto",
  pwf_col = 3,
  pwf_lwd = 2,
  xlab = "Biased Data in <binsize> gene bins.",
  ylab = "Proportion DE",
  ...
)
}
\arguments{
\item{pwf}{A data frame with 3 columns named DEgenes, bias.data & pwf and
row names giving the gene names.  Usually generated by \code{\link{nullp}}.}

\item{binsize}{Calculate and plot the fraction of genes that are DE in bins
of this size.  If set to "auto" the best binsize for visualization is
attempted to be found automatically.}

\item{pwf_col}{The colour of the probability weighting function}

\item{pwf_lwd}{The width of the probability weighting function}

\item{xlab}{The x-axis label. \code{<binsize>} is replaced by the binsize
used.}

\item{ylab}{The y-axis label.}

\item{...}{Extra arguments that are passed to plot.}
}
\value{
Nothing is returned.
}
\description{
Plots the Probability Weighting Function created by \code{\link{nullp}} by
binning together genes.
}
\details{
This function is almost always called using the output from the
\code{\link{nullp}} function.  However, it can be used to visualize the
length (or any other type of quantifiable) bias in ability to detect DE in a
data set.  The \code{pwf} argument needs to be a data frame with 3 columns
each containing numeric entries (although NAs are permitted in the bias.data
and pwf columns), which must be named "DEgenes", "bias.data" and "pwf",
although they can appear in any order.  The row names are taken to be the
gene names. The DEgenes column should be 0s or 1s where 1 represents a DE
gene, 0 a gene which is not DE.  The bias.data column is a quantification of
the quantity for which there is a bias in detecting DE for the associated
gene, this is usually gene length or the number of counts associated with a
gene.  Finally, the pwf column gives the probability weighting to be applied
for a given gene.
}
\examples{
data(genes)
pwf <- nullp(genes, 'hg19', 'ensGene',plot.fit=FALSE)
plotPWF(pwf,binsize=200)

}
\references{
Young, M. D., Wakefield, M. J., Smyth, G. K., Oshlack, A. (2010)
\emph{Gene ontology analysis for RNA-seq: accounting for selection bias}
Genome Biology Date: Feb 2010 Vol: 11 Issue: 2 Pages: R14
}
\seealso{
\code{\link{nullp}}
}
\author{
Matthew D. Young \email{myoung@wehi.edu.au}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nullp.R
\name{nullp}
\alias{nullp}
\title{Probability Weighting Function}
\usage{
nullp(DEgenes, genome, id, bias.data = NULL, plot.fit = TRUE)
}
\arguments{
\item{DEgenes}{A named binary vector where 1 represents DE, 0 not DE and the
names are gene IDs.}

\item{genome}{A string identifying the genome that \code{genes} refer to.
For a list of supported organisms run \code{\link{supportedGenomes}}.}

\item{id}{A string identifying the gene identifier used by \code{genes}.
For a list of supported gene IDs run \code{\link{supportedGeneIDs}}.}

\item{bias.data}{A numeric vector containing the data on which the DE may
depend.  Usually this is the median transcript length of each gene in bp.
If set to \code{NULL} \code{nullp} will attempt to fetch length using
\code{\link{getlength}}.}

\item{plot.fit}{Plot the PWF or not?  Calls \code{\link{plotPWF}} with
default values if \code{TRUE}.}
}
\value{
A data frame with 3 columns, named "DEgenes", "bias.data" and "pwf"
with the rownames set to the gene names.  Each row corresponds to a gene
with the DEgenes column specifying if the gene is DE (1 for DE, 0 for not
DE), the bias.data column giving the numeric value of the DE bias being
accounted for (usually the gene length or number of counts) and the pwf
column giving the genes value on the probability weighting function. This
object is usually passed to \code{goseq} to calculate enriched categories or
\code{plotPWF} for further plotting.
}
\description{
Calculates a Probability Weighting Function for a set of genes based on a
given set of biased data (usually gene length) and each genes status as
differentially expressed or not.
}
\details{
It is essential that the entire analysis pipeline, from summarizing raw
reads through to using \code{goseq} be done in just one gene identifier
format.  If your data is in a different format you will need to obtain the
gene lengths and supply them to the \code{nullp} function using the
\code{bias.data} argument.  Converting to a supported format from another
format should be avoided whenever possible as this will almost always result
in data loss.

\code{NA}s are allowed in the bias.data vector if you do not have
information about a certain gene.  Setting a gene to \code{NA} is preferable
to removing it from the analysis.

If \code{bias.data} is left as \code{NULL}, \code{nullp} attempts to use
\code{\link{getlength}} to fetch GO category to gene identifier mappings.

It is recommended you review the fit produced by the \code{nullp} function
before proceeding by leaving \code{plot.fit} as \code{TRUE}.
}
\examples{
data(genes)
pwf <- nullp(genes, 'hg19', 'ensGene')

}
\references{
Young, M. D., Wakefield, M. J., Smyth, G. K., Oshlack, A. (2010)
\emph{Gene ontology analysis for RNA-seq: accounting for selection bias}
Genome Biology Date: Feb 2010 Vol: 11 Issue: 2 Pages: R14
}
\seealso{
\code{\link{supportedGenomes}}, \code{\link{supportedGeneIDs}},
\code{\link{goseq}}, \code{\link{getlength}}
}
\author{
Matthew D. Young \email{myoung@wehi.edu.au}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getgo.R
\name{getgo}
\alias{getgo}
\title{Fetch GO categories}
\usage{
getgo(genes, genome, id, fetch.cats = c("GO:CC", "GO:BP", "GO:MF"))
}
\arguments{
\item{genes}{A vector or list of genes to get the associated GO categories.}

\item{genome}{A string identifying the genome that \code{genes} refer to.
For a list of supported organisms run \code{\link{supportedGenomes}}.}

\item{id}{A string identifying the gene identifier used by \code{genes}.
For a list of supported gene IDs run \code{\link{supportedGeneIDs}}.}

\item{fetch.cats}{A vector specifying which categories to fetch the mapping
between category names and genes for.  See details for valid options.}
}
\value{
A list where each entry is named by a gene and contains a vector of
all the associated GO categories.  This can be used directly with the
\code{gene2cat} option in \code{\link{goseq}}.
}
\description{
Obtains all gene ontology (GO) categories associated with a set of genes
using the relevant organism package.
}
\details{
This function attempts to make use of the organism packages
(org.<Genome>.<GeneID>.db) to obtain the mapping between gene ID and GO
categories.  As with \code{\link{getlength}} it is preferable that the same
gene identifier system is used for both summarization and retrieving GO
categories.

Valid options for the \code{fetch.cats} argument are any combination of
"GO:CC", "GO:BP", "GO:MF" & "KEGG".  The three GO terms refer to the
Cellular Component, Biological Process and Molecular Function respectively.
"KEGG" refers to KEGG pathways.

Note that \code{getgo} is a convenience function, designed to make
extracting mappings between GO categories and Gene ID easy.  For less common
organisms and/or gene ID \code{getgo} may fail to return a mapping even when
a legitimate mapping exists in the relevant organism package.  If
\code{getgo} fails, you should always try to build the mapping yourself from
the organism package (if one exists) before deciding that the information is
unavailable.  Further information and examples of this can be found in the
package Vignette.
}
\examples{

genes <- c("ENSG00000124208", 
           "ENSG00000182463", 
           "ENSG00000124201", 
           "ENSG00000124205", 
           "ENSG00000124207")
getgo(genes,'hg19','ensGene')

}
\seealso{
\code{\link{supportedGenomes}}, \code{\link{supportedGeneIDs}},
\code{\link{goseq}}
}
\author{
Matthew D. Young \email{myoung@wehi.edu.au}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getlength.R
\name{getlength}
\alias{getlength}
\title{Retrieves Gene length data}
\usage{
getlength(genes, genome, id)
}
\arguments{
\item{genes}{A vector or list of the genes for which length information is
required.}

\item{genome}{A string identifying the genome that \code{genes} refer to.
For a list of supported organisms run \code{\link{supportedGenomes}}.}

\item{id}{A string identifying the gene identifier used by \code{genes}.
For a list of supported gene IDs run \code{\link{supportedGeneIDs}}.}
}
\value{
Returns a vector of the gene lengths, in the same order as
\code{genes}.  If length data is unavailable for a particular gene NA is
returned in that position.  The returned vector is intended for use with the
\code{bias.data} option of the \code{\link{nullp}} function.
}
\description{
Gets the length of each gene in a vector.
}
\details{
Length data is obtained from data obtained from the UCSC genome browser for
each combination of \code{genome} and \code{id}.  As fetching this data at
runtime is time consuming, a local copy of the length information for common
genomes and gene ID are included in the \pkg{geneLenDataBase} package.  This
function uses this package to fetch the required data.

The length of a gene is taken to be the median length of all its mature,
mRNA, transcripts.  It is always preferable to obtain length information
directly for the gene ID used to summarize your count data, rather than
converting IDs and then using the supplied databases.  Even when two genes
have a one-to-one mapping between different identifier conventions (which is
often not the case), they frequently refer to slightly different regions of
the genome with different lengths.  It is therefore recommended that the
user perform the full analysis in terms of only one gene ID, or manually
obtain their own length data for the identifier used to bin reads by gene.
}
\examples{

genes <- c("ENSG00000124208", 
           "ENSG00000182463", 
           "ENSG00000124201", 
           "ENSG00000124205", 
           "ENSG00000124207")
getlength(genes,'hg19','ensGene')

}
\seealso{
\code{\link{supportedGenomes}}, \code{\link{supportedGeneIDs}},
\code{\link{nullp}}, \pkg{geneLenDataBase}
}
\author{
Matthew D. Young \email{myoung@wehi.edu.au}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/goseq-data.R
\docType{data}
\name{genes}
\alias{genes}
\title{Androgen stimulation of prostate cancer Cell lines.}
\format{
A named vector of ENSEMBL genes, with 1 representing differential
expression.
}
\source{
Determination of tag density required for digital transcriptome
analysis: application to an androgen-sensitive prostate cancer model, 2008,
Li et. al.
}
\description{
This data set gives the RNA-seq data from an experiment measuring the
effects of androgen stimulation on prostate cancer.  Information is given
about all (ENSEMBL) genes for which there was at least one mapping read in
either the treated or untreated RNA-seq experiment.  The \code{edgeR}
package was used to determine which genes were differentially expressed.
The details of the analysis can be found in the \code{goseq} vignette.
}
\examples{

data(genes)
head(genes)

}
\references{
Li, H., Lovci, M. T., Kwon, Y. S., Rosenfeld, M. G., Fu, X. D.,
Yeo, G. W. (2008) \emph{Determination of tag density required for digital
transcriptome analysis: application to an androgen-sensitive prostate cancer
model} Proceedings of the National Academy of Sciences of the United States
of America Date: Dec 23 Vol: 105 Issue: 51 Pages: 20179-84
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/goseq.R
\name{goseq}
\alias{goseq}
\title{goseq Gene Ontology analyser}
\usage{
goseq(
  pwf,
  genome,
  id,
  gene2cat = NULL,
  test.cats = c("GO:CC", "GO:BP", "GO:MF"),
  method = "Wallenius",
  repcnt = 2000,
  use_genes_without_cat = FALSE
)
}
\arguments{
\item{pwf}{An object containing gene names, DE calls, the probability
weighting function. Usually generated by \code{\link{nullp}}.}

\item{genome}{A string identifying the genome that \code{genes} refer to.
For a list of supported organisms run \code{\link{supportedGenomes}}.}

\item{id}{A string identifying the gene identifier used by \code{genes}.
For a list of supported gene IDs run \code{\link{supportedGeneIDs}}.}

\item{gene2cat}{A data frame with two columns containing the mapping between
genes and the categories of interest.  Alternatively, a list where the names
are genes and each entry is a vector containing GO categories associated
with that gene (this is the output produced by \code{\link{getgo}}).  If set
to \code{NULL} goseq attempts to fetch GO categories automatically using
\code{\link{getgo}}.}

\item{test.cats}{A vector specifying which categories to test for over
representation amongst DE genes.  See details for allowed options.}

\item{method}{The method to use to calculate the unbiased category
enrichment scores.  Valid options are "Wallenius", "Sampling" &
"Hypergeometric".  "Hypergeometric" and "Sampling" should almost never be
used (see details).}

\item{repcnt}{Number of random samples to be calculated when random sampling
is used.  Ignored unless \code{method="Sampling"}.}

\item{use_genes_without_cat}{A boolean to indicate whether genes without a
category should still be used. For example, a large number of gene may have
no GO term annotated. If this option is set to FALSE, those genes will be
ignored in the calculation of p-values (default behaviour). If this option
is set to TRUE, then these genes will count towards the total number of
genes outside the category being tested (default behaviour prior to version
1.15.2).}
}
\value{
goseq returns a data frame with several columns.  The first column
gives the name of the category, the second gives the p-value for the
associated category being over represented amongst DE genes.  The third
column gives the p-value for the associated category being under represented
amongst DE genes.  The p-values have not been corrected for multiple
hypothesis testing. The fourth and fifth columns give the number of
differentially expressed genes in the category and total genes in the
category respectively. If any of the categories was a GO term, there will be
two additional columns for the GO term and its ontology.
}
\description{
Does selection-unbiased testing for category enrichment amongst
differentially expressed (DE) genes for RNA-seq data.  By default, tests
gene ontology (GO) categories, but any categories may be tested.
}
\details{
The \code{pwf} argument is almost always the output of the function
\code{\link{nullp}}.  This is a data frame with 3 columns, named "DEgenes",
"bias.data" and "pwf" with the rownames set to the gene names.  Each row
corresponds to a gene with the DEgenes column specifying if the gene is DE
(1 for DE, 0 for not DE), the bias.data column giving the numeric value of
the DE bias being accounted for (usually the gene length or number of
counts) and the pwf column giving the genes value on the probability
weighting function.

\code{goseq} obtains length data from UCSC and GO mappings from the organism
packages (see \code{link{getgo}} and \code{\link{getlength}} for details).
If your data is in an unsupported format you will need to obtain the GO
category mapping and supply them to the \code{goseq} function using the
\code{gene2cat} argument.

To use your own gene to category mapping with \code{goseq}, use the
\code{gene2cat} argument.  This argument takes a data.frame, with one column
containing gene IDs and the other containing the associated categories.  As
the mapping from gene <-> category is in general many to many there will be
multiple rows containing the same gene identifier.  Alternatively,
\code{gene2cat} can take a list, where the names are the genes and the
entries are the GO categories associated with the genes.  This is the format
produced by the \code{\link{getgo}} function and is more space efficient
than the data.frame representation.

If \code{gene2cat} is left as \code{NULL}, \code{goseq} attempts to use
\code{\link{getgo}} to fetch GO category to gene identifier mappings.

The PWF is usually calculated using the \code{\link{nullp}} function to
correct for length bias. However, \code{goseq} will work with any vector of
weights.  Any bias can be accounted for so long as a weight for each gene is
supplied using this argument.  \code{NA}s are allowed in the "pwf" and
"bias.data" columns of the PWF data frame (these usually occur as a result
of missing length data for some genes).  Any entry which is \code{NA} is set
to the weighting of the median gene.

Valid options for the \code{test.cats} argument are any combination of
"GO:CC", "GO:BP", "GO:MF" & "KEGG".  The three GO terms refer to the
Cellular Component, Biological Process and Molecular Function respectively.
"KEGG" refers to KEGG pathways.

The three methods, "Wallenius", "Sampling" & "Hypergeometric", calculate the
p-values as follows.

"Wallenius" approximates the true distribution of numbers of members of a
category amongst DE genes by the Wallenius non-central hypergeometric
distribution.  This distribution assumes that within a category all genes
have the same probability of being chosen.  Therefore, this approximation
works best when the range in probabilities obtained by the probability
weighting function is small. "Wallenius" is the recommended method for
calculating p-values.

"Sampling" uses random sampling to approximate the true distribution and
uses it to calculate the p-values for over (and under) representation of
categories. In practice, its use quickly becomes computationally prohibitive
because \code{repcnt} would need to be set very high for most applications.

CAUTION: "Hypergeometric" should NEVER be used for producing results for
biological interpretation.  If there is genuinely no bias in power to detect
DE in your experiment, the PWF will reflect this and the other methods will
produce accurate results.

"Hypergeometric" assumes there is no bias in power to detect differential
expression at all and calculates the p-values using a standard
hypergeometric distribution.  Useful if you wish to test the effect of
selection bias on your results.
}
\examples{

data(genes)
pwf <- nullp(genes,'hg19','ensGene')
pvals <- goseq(pwf,'hg19','ensGene')
head(pvals)

}
\references{
Young, M. D., Wakefield, M. J., Smyth, G. K., Oshlack, A. (2010)
\emph{Gene ontology analysis for RNA-seq: accounting for selection bias}
Genome Biology Date: Feb 2010 Vol: 11 Issue: 2 Pages: R14
}
\seealso{
\code{\link{nullp}}, \code{\link{getgo}}, \code{\link{getlength}}
}
\author{
Matthew D. Young \email{myoung@wehi.edu.au}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/goseq-pkg.R
\docType{package}
\name{goseq-pkg}
\alias{goseq-package}
\alias{goseq-pkg}
\title{goseq}
\description{
Detects Gene Ontology and/or other user defined categories which are 
over/under represented in RNA-seq data.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/federicomarini/goseq}
  \item Report bugs at \url{https://github.com/federicomarini/goseq/issues}
}

}
\author{
\strong{Maintainer}: Federico Marini \email{marinif@uni-mainz.de} (\href{https://orcid.org/0000-0003-3252-7758}{ORCID}) [contributor]

Authors:
\itemize{
  \item Matthew Young \email{my4@sanger.ac.uk}
  \item Nadia Davidson \email{nadia.davidson@mcri.edu.au}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makespline.R
\name{makespline}
\alias{makespline}
\title{Monotonic Spline}
\usage{
makespline(x, y, newX = NULL, nKnots = 6, lower_bound = 10^-3)
}
\arguments{
\item{x}{The predictor variable.}

\item{y}{The response variable.  Must be the same length as \code{x}.}

\item{newX}{The points at which to return the value on the fitted spline.
If not specified \code{x} is used.}

\item{nKnots}{The number of knots to use in fitting the spline.}

\item{lower_bound}{The spline cannot drop below this value.}
}
\value{
Returns a vector of values containing the value of the fit at each
point \code{newX}.
}
\description{
Fits a monotonic cubic spline to the data provided, using the penalized
constrained least squares method from the \code{mgcv} package.
}
\details{
This uses the \code{pcls} function from the \pkg{mgcv} package to produce
the fit.  The monotonicity constraint is enforced using \code{mono.con} from
the same package. The \code{lower_bound} argument is only used on the rare
occasions when the fitting function becomes negative or arbitrarily close to
zero.  If this does occur \code{lower_bound} is added everywhere to ensure
that no one length is given essentially infinite weighting.
}
\examples{
y <- c( rbinom(50,p=0.4,size=1), rbinom(50,p=0.6,size=1) )
x <- 1:100
plot(x,y)
p <- makespline(x,y)
lines(x,p)

}
\references{
Package \pkg{mgcv}.  In particular this function is a
modification of an example given in the man page for \code{pcls}.
}
\author{
Matthew D. Young \email{myoung@wehi.edu.au}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pp.R
\name{pp}
\alias{pp}
\title{Prints progress through a loop}
\usage{
pp(total, count, i = i)
}
\arguments{
\item{total}{total number of iterations}

\item{count}{current iteration}

\item{i}{index of the loop}
}
\value{
message indicating the progress
}
\description{
Prints progress through a loop
}
